//
// MiniCadHookIntf.h
//
// Copyright 1998-2000 Nemetschek North America
// All Rights Reserved.
//
// This file defines constants for the interface between the VectorWorks application and the SDK Plug-ins.
// This file is shared between the VectorWorks application and SDK Plug-ins code.
//

#ifndef _MINICADHOOKINTF_
#define _MINICADHOOKINTF_

#pragma once

#if TARGET_RT_MAC_CFM
	#include <MixedMode.h>
#endif

#include "SharedDefines.h"
#include "GSTypes.h"

#if defined(_MINICAD_EXTERNAL_) && !defined(__EXTERNDEFINES__) && __MWERKS__
	#error GSBUG.h, GSTEST.h, or GSRELEASE.h MUST be included in your C/C++ compiler preferences when compiling under Metrowerks.
#endif


/////////////	-----------------------------------------------
///////////		BEGIN MENU DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////		-----------------------------------------------
///////
/////
///
//
//
// 	Tool Definition Procedure (TDP) parameter descriptions:
//
//																Message1:						Message2:					Result:
//	Action/Event			When called							Gets							Gets						Returns
//	-----------------------------------------------------------------------------------------------------------------------------------------------------------------
//	kToolInitGlobals		When overlay is activated			0								0							0 or kToolCouldNotInitialize
//	kToolDisposeGlobals		When overlay is deactivated			0								0							0
//	kToolDoSetup			When tool is selected or resumed	0								0							0 or kToolCouldNotSetUp
//																or kSetupSetdownPauseRestoreFlag 
//	kToolDoSetDown			When tool is deselected	or paused	0								0							0
//																or kSetupSetdownPauseRestoreFlag
//	kToolGetCursor			When tool selected and curs in doc	0								0							cursorID or 0 to use default
//	kToolDoInterface		When tool is clicked in drawing		Ptr to WorldPt of click			Ptr to Point of click		0 or kToolDidNotChangeDoc and/or kToolSwitchToCursor 								
//	kToolDoModeEvent		When mouse is clicked in mode bar	(long) mode group				HiWrd = old, LoWrd = new	0
//	kToolDoDoubleClick		When tool is double-clicked			0								0							0
//	kToolDoDrawScrMod
//	kToolDoUndrawScrMod
//	kToolPointAdded			When mouse is clicked in drawing																0, kToolCancel, kToolCompleted, kToolSwitchToCursor		
//	kToolMouseMove			When mouse moves in drawing																		0, kToolCancel, kToolCompleted, kToolSwitchToCursor		
//	kToolDrawingDoubleClick	When double-click in drawing																	kToolCompleted, kHandledDocumentDoubleClick
//	kToolDraw 				Whenever tool needs to draw																		0, kToolCancel, kToolCompleted, kToolSwitchToCursor		
//	kToolHandleComplete 	When tool is done getting points																0, kToolCancel, kToolSwitchToCursor		
//	kToolGetStatus 			When app needs tool's status																	kToolWaitingForFirstPoint, kToolCollectingPoints, kToolWaitingForMouseUp, kToolCompleted, kToolSwitchToCursor
//	kToolPointRemoved       When app removes a point			0
//																or kClearingAllToolPoints									0, kToolCancel, kToolCompleted, kToolSwitchToCursor		
//	kToolGenericStateChange After every key event. When the   	0 								0							0
//																or kToolGenericStateChangeKey 
//																or kToolGenericStateChangeView

// Tool action/event  constants 
//	const long kInternalAction1			= -101L;	// internal use only. external tools will not recieve this.

typedef long ToolPropID;


/////////////////////////////
//	Extended Tool Properties
const ToolPropID kToolXPropSelectionChange = 1;		// tool wants selection change events.
const ToolPropID kToolXPropUnsupported = 2;			// not supported
const ToolPropID kToolXPropWantsMoveSelection2D = 3;		// tool wants kToolMoveSelection2DEvent event with nudge and move 2D commands.
const ToolPropID k2DToolEnabledInStackedLayerMode = 4;		// 2D tool wants to be enabled in Unified View.
const ToolPropID kToolXPropSetUnrestrictedMode	= 5;		// set unrestricted scaling mode to the 2D selection tool

/////////////////////////////

typedef long ToolEventID;

//	Tool actions/Events
const ToolEventID kToolInitGlobals			= 1L;
const ToolEventID kToolDisposeGlobals		= 2L;
const ToolEventID kToolDoSetup				= 3L;
const ToolEventID kToolDoSetDown			= 4L;
const ToolEventID kToolGetCursor			= 5L;
const ToolEventID kToolDoInterface	 		= 6L;	// to be removed
const ToolEventID kToolDoModeEvent			= 7L;
const ToolEventID kToolDoDoubleClick		= 8L;
const ToolEventID kToolDoDrawScrMod			= 9L;
const ToolEventID kToolDoUndrawScrMod		= 10L;
const ToolEventID kToolPointAdded			= 100L;
const ToolEventID kToolMouseMove			= 101L;
const ToolEventID kToolDrawingDoubleClick	= 102L;
const ToolEventID kToolDraw 				= 103L;
const ToolEventID kToolHandleComplete 		= 104L;
const ToolEventID kToolGetStatus 			= 105L;
const ToolEventID kToolPointRemoved			= 106L;
const ToolEventID kToolGenericStateChange	= 107L;			// 	Action called when any program state change
													//	occurs that might affect a tool.  Tools that store cached 
													//	data might listen for this message to update their cache.
													//	Newly written tools should not find any need for this 
													//	action as cached data is usually not required - it is provided 
													//	to ease tool modernization.  An example where this action is
													//	observed is the Wall tool.  It is a very complicated tool
													//	that maintains a cache representing the current walls.  After a 
													//	key is pressed to lock a value in the data bar kToolGenericStateChange
													//	is called and the wall tool updates its cache so it can be redrawn 
													//	correctly.  This would not be necessary if the tool's kToolDraw message
													//	was able, like most tools, to draw itself with the tool points.
													//	GENERAL RULE:  Avoid using cached data to handle kToolDraw action calls.
													//	Use the current tool point data.
													
//	Above actions sent to tool prior to VectorWorks 11.



//	Actions/Events below can be sent to Tool Definition Procedure (TDP) with Extended Properties.
const ToolEventID kToolOnInitXProperties 	= 108L;
const ToolEventID kToolOnSelectionChange   	= 109L;															
const ToolEventID kToolOnMoveSelection2D	= 110L;														
const ToolEventID kToolCustomSubtypeInsert	= 111L;		//	Object has declared itself in the "PExt" resource to have EPluginObjectSubType 
														//	equal to kCustomSubtype.  kCustomSubtype objects are provided a tool that calls 
														//	the object definition procedure main with kToolCustomSubtypeInsert events.  
														//	kToolCustomSubtypeInsert events wrap tool events in a ToolActionHandlerParams structure.  
														//	The message paramameter is a pointer to this structure.
const ToolEventID kToolOnIdleEvent			= 112L;		// 				
const ToolEventID kToolOnCustomBarEvent     = 113L;		// The Tool has installed a Custom Editable data bar.  	CustomBarEventData is supplied in the message1 parameter.			

//	Force the imaging flags for release build

#define ENHANCED_IMAGING	1
#define UPDATED_SELECTION	1
#define ANIMATED_SELECTION	1
#define ANIMATED_SEL_SPEC	0

#if UPDATED_SELECTION
	#define SUPPORT_OBJECT_GRIPS
	//	SUPPORT_OBJECT_GRIPS goes along with UPDATED_SELECTION.  When defined
	const ToolEventID kToolProvideObjectGrips      = 114;		// The Tool wants the application to draw grips for the current selected objects.
#endif

// Actions/Events below are sent to Tools
const ToolEventID kToolDisplayContextualHelp	= 115L;		// The help system would like to know if the tool has a contextual help identifier. It is passing in a pointer to a TXString that should be filled in with the help identifier.

const ToolEventID kToolInitByObject				= 116L;		// Send to the tool with MCObjectHandle as message1.
															// This notifies the tool that is activated from the specified parametric object
															// allowing the tool to initialize itself for creating objects of this type
#ifdef GS_USE_NEW_SELECTION_TOOL
const ToolEventID kToolOnViewChange				= 117L;		// NNA Internal USE
#endif

const ToolEventID kOnUpdate2D3DCursor			= 118L;		// ToolDef Decides which cursor to use action was handled by a ToolDef


//----------------------------------------------------------------------------
// Default tool informatoin
// ISDK::CallStandardDefaultTool type
enum EDefaultToolType
{
	eDefaultToolType_Point,
	eDefaultToolType_Line,
	eDefaultToolType_Rect,
	eDefaultToolType_Poly2D,
	eDefaultToolType_Poly3D,
};

// Default point tool
// Allow us to use the default insertion tool as base for all point like tools
struct SDefaultPointToolInfo
{
	short fModeGroup;			// radio group
	short fAllowInsertGroup;	// button group (single button)
	short fAlignmentGroup;		// radio group

	// ---------
	// fModeGroup:
	//		insertModeGroup choice indices
	short fRegularInsChoiceIndex;
	short fOffsetInsChoiceIndex;
	short fPickUpChoiceIndex;

	// ---------
	// fAlignmentGroup:
	//		alignmentGroup choice indices
	short fLeftChoiceIndex;
	short fCenterChoiceIndex;
	short fRightChoiceIndex;
	short fNaturalChoiceIndex;

	SDefaultPointToolInfo() {
		fModeGroup = fAllowInsertGroup = fAlignmentGroup = -1;
		fRegularInsChoiceIndex = fOffsetInsChoiceIndex = fPickUpChoiceIndex = -1;
		fLeftChoiceIndex = fCenterChoiceIndex = fRightChoiceIndex = fNaturalChoiceIndex = -1;
	}
};

// Default line tool
// Allow us to use the default insertion tool as base for all line like tools
struct SDefaultLineToolInfo
{
	short	fModeGroup;	// radio group

	// ---------
	// fModeGroup:
	//		line mode choice indices
	short fConstrainedChoiceIndex;
	short fUnconstrainedChoiceIndex;

	SDefaultLineToolInfo() {
		fModeGroup = fConstrainedChoiceIndex = fUnconstrainedChoiceIndex = -1;
	}
};

// Default rect tool
// Allow us to use the default insertion tool as base for all rect like tools
struct SDefaultRectToolInfo
{
	short	fModeGroup;			// radio group

	// ---------
	// fModeGroup:
	//		rect mode choice indices
	short fCenterLineChoiceIndex;
	short fEdgePlacementChoiceIndex;

	SDefaultRectToolInfo() {
		fModeGroup = fCenterLineChoiceIndex = fEdgePlacementChoiceIndex = -1;
	}
};

// Default 2D poly tool
// Allow us to use the default insertion tool as base for all 2D poly like tools
struct SDefault2DPolyToolInfo
{
	short	fModeGroup;			// radio group
	short	fFiletRadiusGroup;	// button group (single button)

	// ---------
	// fModeGroup:
	//		2D poly mode choice indices
	short fCornerVertChoiceIndex;
	short fBezierVertChoiceIndex;
	short fCubicVertChoiceIndex;
	short fTangetChoiceIndex;
	short fPointOnArcChoiceIndex;
	short fArcVertChoiceIndex;

	SDefault2DPolyToolInfo() {
		fModeGroup = fFiletRadiusGroup = -1;
		fCornerVertChoiceIndex = fBezierVertChoiceIndex = fCubicVertChoiceIndex = -1;
		fTangetChoiceIndex = fPointOnArcChoiceIndex = fArcVertChoiceIndex = -1;
	}
};

// Default 3D poly tool
// Allow us to use the default insertion tool as base for all 3D poly like tools
struct SDefault3DPolyToolInfo
{
	short	fModeGroup;			// radio group
	short	fNURBSDegSetGroup;	// button group (single button)

	// ---------
	// fModeGroup:
	//		3D poly mode choice indices
	short fNURBSByInterpChoiceIndex;
	short fNURBSByCtrlPtChoiceIndex;

	SDefault3DPolyToolInfo() {
		fModeGroup = fNURBSDegSetGroup = -1;
		fNURBSByInterpChoiceIndex = fNURBSByCtrlPtChoiceIndex = -1;
	}
};

// call back message for ISDK::CallDefaultTool for notifying the default tool
// the message expects (message1 = MCObjectHandle) to the new object preview
const ToolEventID kDefaultTool_UpdatePreviewObject	= -100;

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//	BEGIN Custom Editable DataBar types and constants:
//  

//---------------------------------------------------------------
//---------------------------------------------------------------
struct CustomBarEventData
//
//	For tools, the messgae1 parameter of the tool definition procedure contains a CustomBarEventData
//	type for kToolCustomBarEvent action calls.
//
{
	long 	fCustomBarEventSelector;
	long 	fData;
};


////////////////////////////////////////
//
//	kToolCustomBarEvent come with message1 data of type CustomBarEventData.
//	CustomBarEventData::fCustomBarEventSelectors:
//  
const long kCustomBarSetup = 1;			// sent when a CustomBar is installed with GS_CustomBarInstall
const long kCustomBarSetdown = 2;		// sent when a CustomBar is replaced with another databar
const long kCustomBarUpdateValues = 3;	// sent with each new cursor/mouse position so CustomBar client and provide new field values.
const long kCustomBarLockValue = 4;		// sent when a user lockes a field of a custom bar	
const long kCustomBarUnlockValue = 5;	// sent when a user unlockes a field of a custom bar	

const short kModeBarOptionsIconID = 5010;

//	END Custom Editable DataBar types and constants:
//  
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

											
													
//////////////////////////
//
// Tool message1 constants.
const long kSetupSetdownPauseRestoreFlag = 1;		//	Sent in the Message1 parameter with kToolDoSetup and kToolDoSetDown action calls.
													//	This message allows action handling to distinguish the Setup and SetDown action calls.
													//	When tools are being paused and restored they are sent this value in the message1 
													//	parameter.
													  
const long kToolGenericStateChangeKey = 1;			//	Sent in the Message1 parameter with kToolGenericStateChange action call.
													//	A key has been pressed. See kToolGenericStateChange description above.
const long kToolGenericStateChangeView = 2;			//	Sent in the Message1 parameter with kToolGenericStateChange action call.
													//	The current view has been changed. See kToolGenericStateChange description above.

const long kClearingAllToolPoints = 1;				// ToolDef's message1 parameter when action==kToolPointRemoved.

// Tool return constants

const long kToolDidNotChangeDoc			= 1L;		//	observed after kToolHandleComplete action handler call.  It prevents 
													//	the "dirty"flag for the document from being set. 
const long kToolCouldNotInitialize		= -1L;		//	Observed after kToolInitGlobals action handler.  Return this if the tool
													//	failed to initialize. 
const long kToolCouldNotSetUp			= -1L;		//	Observed after kToolDoSetup action handler.  Return this if the tool fails to set up.
const long kToolSwitchToCursor			= 2L;		//	cancel, remove all tool points, and switch to cusor tool
const long kHandledDocumentDoubleClick 	= 4L;		// 	Observed after kToolDrawingDoubleClick action handler.  Return this if the tool's
													//	kToolDrawingDoubleClick action handler wants the second click entered as a tool point.
const long kToolWaitingForFirstPoint	= 5L;		// 	Observed after kToolGetStatus action handler.  Return this if tool is 
													//	waiting for first point.
const long kToolCollectingPoints		= 6L;		// 	Observed after kToolGetStatus action handler.  Return this if tool is still
													//	collecting points.
const long kToolWaitingForMouseUp		= 7L;		// 	Observed after kToolGetStatus action handler.  Tool returns this if it 
													//	wants the next mouse up as a tool point.  
const long kToolCompleted				= 8L;		// 	Observed after kToolGetStatus action handler.  Will cause VW to call 
													//	the plug-in with kToolHandleComplete action
const long kToolCancel					= 9L;		// 	removes all tool points

const long kToolEventHandled   = 1; 				// 	observed by kToolMoveSelection2DEvent and kOnAttributeMove2D event callers.
const long kToolHandledViewChange = kToolEventHandled;

const long kToolHasContextualHelp		= 10L;

const long kToolDoesNotHandleOnUpdate2D3DCursor   = 0; 	// 	observed by kOnUpdate2D3DCursor event caller.

// THe non-zero return constants need to be unique
const long kToolWants2DCursor   = 11L;					// 	observed by kOnUpdate2D3DCursor event caller.
const long kToolWants3DCursor   = 12L;					// 	observed by kOnUpdate2D3DCursor event caller.
const long kToolHandles2D3DCursor   = 13L;				// 	observed by kOnUpdate2D3DCursor event caller.


#ifdef SUPPORT_OBJECT_GRIPS
const long kToolProvideObjectGripsNO		= 0L;	// observed by the kToolProvideObjectGrips action caller.  THe reshape Handles/Grips WILL NOT be drawn.
const long kToolProvideObjectGripsYES		= 1L;	// observed by the kToolProvideObjectGrips action caller.  THe reshape Handles/Grips WILL not be drawn.
#endif

//
///
/////
///////
/////////       ---------------------------------------------
///////////		END TOOL DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////////	---------------------------------------------







/////////////	-----------------------------------------------
///////////		BEGIN MENU DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////		-----------------------------------------------
///////
/////
///
//
//	
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// 	Menu Definition Procedure
//														Message:					Result:
//	Action					Task							Gets						Returns
//	------------------------------------------------------------------------------------------------------------------------------
//	kMenuInitGlobals		Allocates global data				0								0
//	kMenuAddItems			Fills menu with its items			ChunkInfo Pointer				0
//	kMenuDisposeGlobals		Disposes of global data				0								0
//	kMenuDoInterface		Performs function					ChunkInfo Pointer or 0			kMenuDidNotChangeDoc if necessary
//	kMenuCheckHilite		Hilites & Checks items				ChunkInfo Pointer				true if this call is needed
//	kMenuNotify				Notify menu something has changed	statusID						0
//	kMenuItemEnabled		returns whether item is hilited		Chunk index (if it's a chunk)	0 - use Flags, kMenuItemEnabled, kMenuItemDisabled
//	kMenuAccessCheck		Internal support only
//
//
// Menu action constants
const long kMenuInitGlobals			= 1L;
const long kMenuDisposeGlobals		= 2L;
const long kMenuDoInterface			= 3L;
const long kMenuAddItems			= 4L;
const long kMenuCheckHilite			= 5L;
const long kMenuNotify				= 6L;
const long kMenuItemEnabled			= 7L;
const long kMenuAccessCheck			= 8L;
const long kMenuItemCheck			= 9L;
const long kMenuHandleToolAction	= 10L;			//	When a Menu returns kMenuRunTempTool from its kMenuDoInterface
													//	handler it will then be called with kMenuHandleToolAction actions
													//	to handle standard tool actions.  The tool actions are identical to tool
													//	actions for tool definition procedures.  The tool action parameters
													//	are contained in a MenuToolActionHandlerParams structure.
const long kMenuItemDisplayContextualHelp = 11L;


// Menu return constants
const long kMenuDidNotChangeDoc		= 1L;
const long kMenuHandledEvent		= 2L;
const long kMenuItemIsEnabled		= 3L;
const long kMenuItemIsDisabled		= 4L;
const long kMenuNoSuchItem			= 5L;
const long kMenuItemSecurity		= 6L;
const long kMenuRunTemp2DTool		= 7L;			//	Returned when Menu wants to get Interactive 2D points from the 
													//	documnet.  See kMenuHandleToolAction for more.
const long kMenuRunTemp3DTool		= 8L;			//	Returned when Menu wants to get Interactive 2D points from the 
													//	documnet
const long kMenuItemHasContextualHelp	= 9L;
//
///
/////
///////
/////////       ---------------------------------------------
///////////		END MENU DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////////	---------------------------------------------











/////////////	-----------------------------------------------
///////////		BEGIN OBJECT DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////		-----------------------------------------------
///////
/////
///
//
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//
// 	Object Definition Procedure (ODP) action/Event descriptions:
//
//															Message:					Result:
//	Action						Task							Gets						Returns
//	------------------------------------------------------------------------------------------------------------------------------
//	kParametricInitGlobals		Allocates global data				0								0 or kParametricSetupFailed
//	kParametricRecalculate		Regenerate objects					0								0
//	kParametricPreference		maybe pose preference dialog		0 or parmFormat					0 or kParametricNotImplemented, kParametricPrefOK, kParametricPrefCancel
//	kParametricDisposeGlobals	Disposes of global data				0								0
//	
//	
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------------

typedef long ObjectPropID;

//----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// Plug-in Object Extended Properties descriptions:
//
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
//                         Property ID               //  Has Associated     //  Description:
//                                                   //     Event(s)        //
//----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropEditGroup = 1;           //        NO           //  Object has non-default "Enter Group" behavior.  PIO objects can enter the profile or path group 	
//                                                   //                     //  directly.  See kObjectEditGroupPropertyID property values below for more.
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasLayerScaleDeps = 2;   //        NO           //  Object wants to be reset with when its layer scale changes.
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropSpecialEdit = 3;	     //  DEPENDS on         //
//                                                   //  property value.    // Object has non-default behavior when "Edit..." command is invoked.
//                                                   //                     // Cursor double-click also invokes this "Edit..." behavior.
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropPreference = 4;		     //        YES          // Object Definition Procedure (ODP) handles kParametricPreference event.
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropDirectModeling = 5;		 //        YES          // Object Definition Procedure (ODP) handles handles Direct Modeling events.
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropAttributeTool = 6;		 //        YES          // Object Definition Procedure (ODP) handles handles Attribute Mapping events.
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropPreventWallInsertion = 7;//        NO           // Object does NOT want to be inserted into walls.  Observed by the 2D cursor tool
//                                                   //                     // to prevent dragging into a wall.
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasUIOverride = 8;		 //  DEPENDS on         //
//                                                   //  property value.    // Object Definition Procedure (ODP) adds widgets to Object Info Palette in kObjXPropInitXProperties event handler
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropCustomCursorResize = 9;	 //       YES           //
//       // boolean value                            //                     // The O
//      NNA under development                        //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropAcceptsMarkers = 10;     //                     //
//       // unsigned char                            //                     // 
//                                                   //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropDefaultPropertyUI = 11;	 //         NO          //
//       // unsigned char                            //                     // The bits in this property describe the UI displayed
//       // NNA under development                    //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXHasCustomWidgetVisibilities = 12;//      YES        // kObjOnWidgetPrep
//       // boolean                                  //                     // 
//       NNA under development                       //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXHandlesCommand = 13;	     //                     //
//       // boolean                                  //                     // 
//       NNA under development                       //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXIs2DSurfaceEligable = 14;    //                     //
//       // boolean                                  //                     // 
//       Weiler operations operate on 2D path        //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropRotatesWithVCS = 15;	 //                     //
//       // boolean                                  //                     // 
//       Plug-in objects rotates with plan rotation  //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropCustomHideFactor = 16;	 //         NO          // The parametric object provides custom scale threshold to be used to hide inner objects
//       // double                                   //                     // when displaying the object in a layer or VP
//		 Plug-in provides custom hide factor		 //                     // See documentation for more info.
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXHasCustomWidgetValues = 17;  //         YES         // kObjOnWidgetValueUpdate
//       // boolean                                  //                     // 
//       Plug-in object has custom widget values     //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropAcceptStates = 18;		 //         YES         // The parametric accpets kObjOnAddState event
//       // Boolean                                  //                     // event value is TParametricStateFlags
//		 Plug-in accepts states						 //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropAcceptStatesInternal = 19;//        NO          // The plug-in will receive about to be delete and duplicate (internal) states through kObjXPropAcceptStates
//       // Boolean                                  //                     // Mush have kObjXPropAcceptStates enabled
//		 Plug-in accepts about to be deleted state	 //                     // Note! This is isoleted option to improve speed of plug-ins that are not interested in delte events
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropPreserveContents = 20;	 //         NO          // VectorWorks will not delete the contents of the parametric
//       // Boolean                                  //                     // before reset
//		 VW wont delete contents before reset		 //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasContextMenu = 21;     //         YES         // Sends kObjOnContextMenuInit and kObjOnContextMenuEvent
//       // Boolean                                  //                     // The parametric can build up context menu and respont to context menu items selection
//		 Parametric has custom context menu			 //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasComponents = 22;		 //                     // The parametric defines components
//       // boolean                                  //                     // 
//		 Plug-in has components						 //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropResetBeforeExport = 23;  //         NO          // This plugin will be reset before Print or Export
//       // boolean                                  //                     // 
//		 Reset before Printing or Export			 //                     //
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropShowOthersEditGroup = 24;//         NO          // Show others when editing the selected special group.
//       // Sint8 (kObjectEditGroupPropertyID)		 //                     // - kObjXPropEditGroupProfile - editing only the profile group shows the others;
//		 Reset before Printing or Export			 //                     // - kObjXPropEditGroupPath - editing only the path profile group shows the others;
//													 //						// - kObjXPropEditGroupCustom - editing any special group shows the others;
//													 //						// - kObjXPropEditGroupDefault - undefined (doesnt show anything)
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropRedSymbolIsStyle = 25;	 //         NO          // Red Symbol of this object can be used to drag&drop
//													 //						// over existing instances to copy the attributes
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasAttrMappingGeom = 26; //         YES         // Sends kObjOnGetSpecificGeometry with message is (OnObjectGetSpecificGeometry*)
//       // Boolean                                  //                     // The parametric can provide geometry for attribute mapping tool (use ISDK::GSNewObjectHandle to create the geometry handle)
//		 Parametric has attribute mapping geometry   //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasSectionVPGeom = 27;   //         YES         // Sends kObjOnGetSpecificGeometry with message is (OnObjectGetSpecificGeometry*)
//       // Boolean                                  //                     // The parametric can provide geometry for the section viewport (use ISDK::GSNewObjectHandle to create the geometry handle)
//		 Parametric has section viewport geometry    //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasLayerElevDeps = 28;	 //         NO          // Resets the parametric if the layer scale changes
//       // Boolean                                  //                     // 
//		 Parametric is dependent on the layer elev	 //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kObjXPropHasLayerHeightDeps = 29; //         NO          // Resets the parametric if the layer height changes
//       // Boolean                                  //                     // 
//		 Parametric is dependent on the layer height //                     // 
//---------------------------------------------------//---------------------//-------------------------------------------------------------------------------------------------
const ObjectPropID kLastObjXProp = kObjXPropHasLayerHeightDeps;		// last property ID

//////////////////////////////////////////////////////////////////////
//Property Values:
//

// kObjectEditGroupPropertyID property values:
const unsigned char kObjXPropEditGroupDefault = 0;
const unsigned char kObjXPropEditGroupProfile = 1;
const unsigned char kObjXPropEditGroupPath = 2;
const unsigned char kObjXPropEditGroupCustom = 3;


// kObjectSpecialEditPropertyID property values:
const unsigned char kObjXPropSpecialEditDefault = 0;
const unsigned char kObjXPropSpecialEditCustom = 1;
const unsigned char kObjXPropSpecialEditProperties = 2;
const unsigned char kObjXPropSpecialEditReshape = 3; 

//	kObjXPropDefaultPropertyUI property values:
// NNA under development   
const unsigned char kHideSymbolWidget = 1;
const unsigned char kHidePolyWidget = 2;
const unsigned char kHidePolyAndSymbolRotationWidget = 4;

//  kObjXPropCustomHideFactor property values:
		// Uses document threshold value (varHideCavityFactor) to determine visibility
const double        kCustomHideFactor_DocumentThreshold = (double) -1.0;
		// Uses document threshold value (varHideCavityFactor) to determine visibility
		// Plus for viewports respects the 'Show Wall Components' setting
const double        kCustomHideFactor_ViewportDocumentThreshold = (double) -2.0;

/////////////////////////////
// kObjXPropHasUIOverride Widget constants:
enum EWidgetType {
	kWidgetLongInt			= 1,
	kWidgetBoolean			= 2,
	kWidgetReal				= 3,
	kWidgetText				= 4,
	kWidgetCalculation		= 5,
	kWidgetHandle 			= 6,
	kWidgetCoordDisp		= 7,
	kWidgetPopUp			= 8,
	kWidgetRadio			= 9,
	kWidgetCoordLocX		= 10,
	kWidgetCoordLocY		= 11,
	kWidgetButton			= 12,
	kWidgetStaticText		= 13,
	kWidgetStaticTextParam	= 14,
	kWidgetDimStdPopUp		= 16,
	kWidgetPrecisionPopUp	= 17,
	kWidgetClassesPopup		= 18,
	kWidgetLayersPopup		= 19,
	kWidgetAngleDisp		= 20,
	kWidgetAreaDisp			= 21,
	kWidgetVolumeDisp		= 22,
	kWidgetSeparator		= 100,
	kWidgetSubSelection		= 101,

};
/////////////////////////////

// kObjXPropAcceptsMarkers constants:
const unsigned char kObjXPropAcceptsMarkersNone = 0;
const unsigned char kObjXPropAcceptsMarkersBeginningOnly = 1;
const unsigned char kObjXPropAcceptsMarkersEndOnly = 2;
const unsigned char kObjXPropAcceptsMarkersBoth = 3;

////////////////////////////////////////////	Object definition procedure action/event constants:
typedef long ObjectEventID;

// Plug-in Object action/events constants 
const ObjectEventID kParametricInitGlobals			= 1;
const ObjectEventID kParametricDisposeGlobals		= 2;
const ObjectEventID kParametricRecalculate			= 3;		// The Plug-in Object code should re-create all its component objects using current parameter values.
const ObjectEventID kParametricPreference			= 4;		// Object Definition Procedure (ODP) may optionally present its own preferences dialog, or let VW pose the default dialog.
//
//	Above Actions sent to Object Definition Procedure (ODP) prior to Extended Prperties and Events introduced in VW 10.5 and expanded in VW 11.
//

//
//	Extended Events. These events can target any object type - not just parametric.
//
const ObjectEventID kObjOnInitXProperties		= 5;		// The Plug-in Object may add extended properties.	
const ObjectEventID kObjUnsupported6			= 6;		// unsupported 
const ObjectEventID kObjOnSpecialEditID			= 7;		// The Plug-in Object called with kParametricOnMove when kHasMoveDependancy extended property set 
const ObjectEventID kObjUnsupported8			= 8;		// unsupported

const ObjectEventID kObjOnReshape 				= 9;		// kObjOnDM_## events will be sent inside kObjOnReshape for VW 11.  

const ObjectEventID kObjOnDM_Select 			= 10;	
const ObjectEventID kObjOnDM_Cancel 			= 11;	
const ObjectEventID kObjOnDM_MouseDown 			= 12;	
const ObjectEventID kObjOnDM_Draw 				= 13;	
const ObjectEventID kObjOnDM_Complete 			= 14;	
const ObjectEventID kObjOnDM_GetCursor 			= 15;	
const ObjectEventID kObjOnDM_ModeEvent 			= 16;	
const ObjectEventID kObjOnDM_GetStatus 			= 17;	
const ObjectEventID kObjOnDM_CustomBarEvent		= 18;
const ObjectEventID kObjOnDM_BeginPauseEvent	= 19;
const ObjectEventID kObjOnDM_EndPauseEvent		= 20;
const ObjectEventID kObjOnDM_MouseMove 			= 21;

const ObjectEventID kObjOnAttributeSelect 		= 26;	
const ObjectEventID kObjOnAttributeCancel 		= 27;	
const ObjectEventID kObjOnAttributeMouseDown 	= 28;	
const ObjectEventID kObjOnAttributeDraw 		= 29;	
const ObjectEventID kObjOnAttributeMouseMove 	= 29;		// Meaning has changed for Vw 2011. See T00617
const ObjectEventID kObjOnAttributeComplete 	= 30;	
const ObjectEventID kObjOnAttributeGetCursor 	= 31;	
const ObjectEventID kObjOnAttributeMove2D 	    = 32;	
const ObjectEventID kObjOnAttributeGetStatus 	= 33;	
const ObjectEventID kObjOnAttributeModeEvent 	= 34;

const ObjectEventID kObjOnObjectUIButtonHit		= 35;	// The user has pressed a button in the Properties UI of an object

const ObjectEventID kObjOnCursor_MouseDown 		= 36;	
const ObjectEventID kObjOnCursor_Complete 		= 37;	
const ObjectEventID kObjOnCursor_MouseMove 		= 38;	
const ObjectEventID kObjOnCursor_Draw 			= 39;	
const ObjectEventID kObjOnCursor_Cancel 		= 40;	

const ObjectEventID kObjOnWidgetPrep 			= 41;	
const ObjectEventID kObjOnCommand 		    	= 42;	

const ObjectEventID kObjOnGetToolName			= 43;
const ObjectEventID kObjOnAddState 		    	= 44;

const ObjectEventID kObjOnContextMenuInit		= 45;
const ObjectEventID kObjOnContextMenuEvent		= 46;

const ObjectEventID kObjOnWidgetValueUpdate		= 47;
const ObjectEventID kObjOnEyedropperPrepareCopy	= 48;	// Vlado: this should be renamed
const ObjectEventID kObjOnGetSpecificGeometry	= 49;

const ObjectEventID kObjOnCursor_CustomBarEvent = 50;
const ObjectEventID kObjOnEyedropperAfterCopy	= 51;	// Vlado: this should be renamed

const ObjectEventID kObjLastEventID = kObjOnEyedropperAfterCopy;		// last property ID


// Plug-in Object return constants
const long kParametricNoErr					= 0;		// 
const long kParametricSetupFailed			= -1;
const long kParametricError					= -2;
const long kParametricNotImplemented		= -3;		// The Plug-in did not present its own preference dialog, it is up to the VW application to do it.
const long kParametricPrefOK				= -4;		// The Plug-in presented its own preferences dialog, and the user hit OK.
const long kParametricPrefCancel			= -5;		// The Plug-in presented its own preferences dialog, and the user hit Cancel.
const long kParametricHadError				= -6;		// The Plug-in compiled but had an error during execution.
const long kParametricHadWarning			= -7;		// The Plug-in compiled but had a warning during execution.

const long kObjectEventHandled			  	 = -8;		//
const long kObjectEventFinalized            = -9;       // The event was handled by the plug-in, and the parametric code should not make further changes to the object parameters 
const long kObjectEventNotHandled		    = 0;		// 

const long kObjectOnSpecialOK			   	= 0;		// observed by kObjOnObjectUIButtonHit caller - Undo event is ended
const long kObjectOnSpecialCancel		    = -5;		// observed by kObjOnObjectUIButtonHit caller - Undo event is removed

const long kObjectUIButtonHitOK			   	= 0;		// observed by kObjOnObjectUIButtonHit caller - Undo event is ended
const long kObjectUIButtonHitCancel		    = -5;		// observed by kObjOnObjectUIButtonHit caller - Undo event is removed


//----------------------------------------------------------
//
//	 NNA under development  
//
//	Many object events use ObjectEventCall hierarchy to pass event data
//	in the message parameter of the object definition procedure.
//

//----------------------------------------------------------
struct ObjectEventCall 
//
//	Many events use a ObjectEventCall to supply event data.  For these events the message parameter
//	is a pointer to a  ObjectEventCall structure.
//
//	return kParametricHandledEvent if ODP handled the event, kParametricEventNotHandled if not
{	
	long 	fSpecifier;
	void* 	fData;
};

//----------------------------------------------------------
struct OnObjectWidgetPrepCall : public ObjectEventCall
//
//	Objects with the kObjXHasCustomWidgetVisibilities property have their Object definition procedure called with
//	the kObjOnWidgetPrep event.  The message parameter is a pointer to this structure.  
//
//	fData is a pointer to an array of WidgetInfoType (s)
//	fSpecifier contains the number of WidgetInfoType (s) in the fData
//	fpWidgetProvider is (IWidgetsProvider*) for widget update
//
//	return kParametricHandledEvent if ODP handled the event, kParametricEventNotHandled if not.
{	
	void*	fpWidgetProvider;
};

//----------------------------------------------------------
struct OnObjectWidgetValueCall : public ObjectEventCall
//
//	Objects with the kObjXHasCustomWidgetValues property have their Object definition procedure called with
//	the kObjOnWidgetValueUpdate event.  The message parameter is a pointer to this structure. 
//
//	fData - is (IWidgetsProvider*) 
//	fSpecifier - 0
//	fChangedWidgetID - the ID of the widget that has cahged if it is kOnTranslateToData, otherwise = 0
//	fTranslateType - determine the reason of call
//	fResult - is the result of the event; determine if VW should execute the default translating of the values
{
	enum ETranslateype {
		kOnTranslateToView,		// values are moved from data to the view
		kOnTranslateToData,		// values are moved from view to the data
	};

	Sint32			fChangedWidgetID;
	ETranslateype	fTranslateType;
};

//----------------------------------------------------------
struct ObjectCommand : public ObjectEventCall
//
//	Objects with the kObjXHandlesCommand property have their Object definition procedure called with
//	the kObjOnCommand event.  The message parameter is a pointer to this ObjectCommand structure.  
//	Use the fSubtype to determine the type of data in  fData.
//
//	return kParametricHandledEvent if ODP handled the event, kParametricEventNotHandled if not
{	
	enum ECommandType { 	
	
		kOnSymetricScale = 0, 	//	for this fSubtype fData is a double* that is the scale
								// 	being applied to the object.
								
		kOnMatrixTransform = 1, //	for this fSubtype fData is a TransformMatrix* that is the
								//	transform being applied to the object
		
		kOnDrawSelected = 2,	// for this fSubtype fData is zero
		
		//	The object is selected and the multiple reshape tool has completed the 
		//	marquee drag.
		kOnMultipleReshapeComplete = 3,	// for this fSubtype fData is a pointer to
								// a MultipleReshapeEventData strucutre

		//	The object is selected and the multiple reshape tool is in the middle of
		//	a marquee drag.
		kOnMultipleReshapeDraw = 4	// for this fSubtype fData is a pointer to
								// a MultipleReshapeEventData strucutre
	};
};

//----------------------------------------------------------
struct ObjectState : public ObjectEventCall
// 
//	Objects with the kObjXPropAcceptStates property have their Object definition procedure called with
//	the kObjOnAddState event.  The message parameter is a pointer to this ObjectState structure.  
//	Use the fSpecifier to determine the type of data in  fData.
//
{
	enum EStateType {
		kCreatedReset,				// fData => NULL
		kMovedReset,				// fData => (const ObjectStateData_Position*)
		kRotatedReset,				// fData => (const ObjectStateData_Position*)
		kParameterChangedReset,		// fData => (const ObjectStateData_ParamChanged*)
		kObjectChangedReset,		// fData => (const ObjectStateData_ObjectChanged*)
		kLayerChangedReset,			// fData => (const ObjectStateData_LayerChanged*)
		kExitFromEditGroup,			// fData => (const ObjectStateData_ExitFromEditGroup*)
		kNameChanged,				// fData => (const ObjectStateData_ObjectNameChanged*)
		kObjectUndoRestore,			// fData => NULL
		kObjectUndoRemove,			// fData => NULL
		kPlanarRefChanged,			// fData => (const ObjectStateData_ObjectPlanarRefChanged*)
		kBeforeExportReset,			// fData => NULL
		kObjectUndoModify,			// fData => NULL
	};
};

//----------------------------------------------------------
struct ObjectContextMenuEvent : public ObjectEventCall
// 
//	Objects with the kObjXPropHasContextMenu property have their Object definition procedure called with
//	kObjOnContextMenuInit  or kObjOnContextMenuEvent event.  The message parameter is a pointer to this ObjectContextMenuEvent structure.  
//
//	fData => (IContextMenuProvider*)
//	fSpecifier = 0;
//	fActiveMenuID is the active menu identifier when the menu item is selected
//
{
	long 	fActiveMenuID;
};

//----------------------------------------------------------
struct ObjectGetSpecificGeometryCall : public ObjectEventCall
// kObjOnGetSpecificGeometry event
//
// requests specific reometry from the parametric
// OnObjectGetSpecificGeometry::ESpecifier	type	= (OnObjectGetSpecificGeometry::ESpecifier) fSpecifier;
{
	enum ESpecifier {
		eSpecifier_MappingGeometry,
		eSpecifier_SectionVPGeometry,
	};

	long	fhGeometry;	// return a handle to the geometry here use (ISDK::GSNewObjectHandle to create the geometry handle)
};


///
/////
///////
/////////       -------------------------------------------------------
///////////		END OBJECT DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////////	------------------------------------------------------




/////////////	-----------------------------------------------------------------
///////////		BEGIN PLUGIN LIBRARY DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////		-----------------------------------------------------------------
///////
/////
///
//
//	Action constants
//	Actions >= 0 are reserved for the Plug-in routine implementation calls.
//
const long kPluginLIBInitGlobals = -1;
const long kPluginLIBDisposeGlobals = -2;
const long kVCOMRegisterInterfaces = -3;
const long kVCOMQueryInterface = -4;
const long kVCOMReleaseInterface = -5;

//	Return Flags for kPluginLIBInitGlobals action returns
//
enum PluginLibraryInitFlags { 			
	kFailedToInitLibFlag = 1,
	kDoNotOpenRsrcFileWithCallFlag = 2,
	kUseSingleUserDataFlag = 4,
	kAddCodeReference = 8,	// This plugin library remains loaded until until reference count drops to zero.
	kProvidesInterfacePtrs = 16	// 
						
};
//	Return Flags for kPluginLIBDisposeGlobals action returns
//
enum PluginLibraryDisposeFlags { 			
	kReleaseCodeReference = 8	// This plugin library returned kAddCodeReference from its kPluginLIBInitGlobals handler. 					
};

//	Return constants for kPluginLIBInitGlobals action returns
//
const long kPluginLIBDefaultReturnVal = 0;
const long kPluginLIBFailedToInitialize = kFailedToInitLibFlag;
const long kPluginLIBDoNotOpenRsrcFile = kDoNotOpenRsrcFileWithCallFlag;
const long kPluginLIBDUseSingleUserData = kUseSingleUserDataFlag;
//
///
/////
///////
/////////       ---------------------------------------------------------------
///////////		END PLUGIN LIBRARY DEFINITION PROCEDURE PARAMETERS DESCRIPTIONS -------------------------------------------------
/////////////	---------------------------------------------------------------






/////////////	-----------------------------------------------------------------
///////////		BEGIN PLUGIN LIBRARY VCOM SUPPORT -------------------------------------------------
/////////		-----------------------------------------------------------------
///////
/////
///
//

// ---------------------------------------------------------------------------------------------------
#ifdef _WINDOWS
# define VCOM_CALLTYPE			__stdcall
#else
# define VCOM_CALLTYPE
#endif

// ---------------------------------------------------------------------------------------------------
struct VWIID
{
    unsigned long  data1;
    unsigned short data2;
    unsigned short data3;
    unsigned char  data4[8];

	VWIID& operator = (const VWIID& iid) {
		memcpy( this, &iid, sizeof(VWIID) );
		return *this;
	}

	bool operator == (const VWIID& iid) const {
		return memcmp( this, &iid, sizeof(VWIID) ) == 0;
	}

	bool operator < (const VWIID& iid) const {
		bool	isLess	= true;
		bool	isEqual	= true;

		// data1
		if ( isEqual ) {
			isEqual		= isEqual && data1 == iid.data1;
			if ( ! isEqual ) { isLess	= (data1 < iid.data1); }

			// data2
			if ( isEqual ) {
				isEqual		= isEqual && data2 == iid.data2;
				if ( ! isEqual ) { isLess	= (data2 < iid.data2); }

				// data3
				if ( isEqual ) {
					isEqual		= isEqual && data3 == iid.data3;
					if ( ! isEqual ) { isLess	= (data3 < iid.data3); }

					// data4
					for(int i=0; i<8 && isEqual; i++) {
						isEqual		= isEqual && data4[i] == iid.data4[i];
						if ( ! isEqual ) { isLess	= (data4[i] < iid.data4[i]); }
					}
				}
			}
		}

		return isLess && ! isEqual;
	}
	void SetToNULL(void) { memset(this, 0, sizeof(VWIID)); }
	bool IsNULL(void) { VWIID compare; compare.SetToNULL(); return(*this == compare);}
};

// ---------------------------------------------------------------------------------------------------
typedef	VWIID	VWGroupID;

// ---------------------------------------------------------------------------------------------------
// We define Dynamic Attribute modificator so VCOM and other C++ interfaces
//		could participate in dynamic_cast operator on Mac.
// This is a bug (or issue) in Mac compiler that causes dynamic_cast operator to fail
//		if the interface instance is from local memory of a DLL!
#ifdef _WINDOWS
# define DYNAMIC_ATTRIBUTE
#else
# define DYNAMIC_ATTRIBUTE __attribute__((visibility("default")))
#endif

// ---------------------------------------------------------------------------------------------------
// Base interface for all interfaces in VCOM
class DYNAMIC_ATTRIBUTE IVWUnknown
{
public:
	virtual							~IVWUnknown( void )		{}
	virtual long VCOM_CALLTYPE		AddRef() = 0;
	virtual long VCOM_CALLTYPE		Release() = 0;
	
};

// ---------------------------------------------------------------------------------------------------
// Base interface for single instance interfaces in VCOM
class DYNAMIC_ATTRIBUTE IVWSingletonUnknown : public IVWUnknown
{
public:
};

// ---------------------------------------------------------------------------------------------------
// VCOM errors
typedef	long		VCOMError;

const VCOMError		kVCOMError_NoError			= 0;
const VCOMError		kVCOMError_Failed			= 1;
const VCOMError		kVCOMError_False			= 2;
const VCOMError		kVCOMError_OutOfMemory		= 3;
const VCOMError		kVCOMError_NotInitialized	= 4;
const VCOMError		kVCOMError_NoInterface		= 5;
const VCOMError		kVCOMError_NotImplemented	= 6;
const VCOMError		kVCOMError_DuplicateIID		= 10;
const VCOMError		kVCOMError_NoModule			= 11;
const VCOMError		kVCOMError_NoInstance		= 12;
const VCOMError		kVCOMError_Unexpected		= 13;
const VCOMError		kVCOMError_InvalidArg		= 15;

#define		VCOM_SUCCEEDED(x)		(x == kVCOMError_NoError)
#define		VCOM_FAILED(x)			(x != kVCOMError_NoError)

// ---------------------------------------------------------------------------------------------------
// working with interfaces
VCOMError	GS_InitializeVCOM(void* cbpParam);
VCOMError	GS_VWRegisterInterface(const VWIID& iid, void* argTable);
VCOMError	GS_VWRegisterInterface(const VWIID& iid, const VWIID& iidContainer, void* argTable);
VCOMError	GS_VWQueryInterface(const VWIID& iid, IVWUnknown** ppInterface);
void		GS_VWNotifyDeleteInterface(IVWUnknown* pInterface);
void		GS_VWLoadInterfaceResource(IVWUnknown* pInterface);
void		GS_VWUnloadInterfaceResource(IVWUnknown* pInterface);
size_t		GS_VWGetContainerInterfacesCount(const VWIID& iidContainer);
VWIID		GS_VWGetContainerInterface(const VWIID& iidContainer, size_t index);

// ---------------------------------------------------------------------------------------------------
// tools
template<class T> class VCOMPtr
{
public:
				VCOMPtr()						{ fPtr = NULL; }
				VCOMPtr(T* ptr)					{ fPtr = ptr;						if ( fPtr ) { fPtr->AddRef(); } }
				VCOMPtr(const VWIID& iid)		{ fPtr = NULL;						this->Query( iid ); }
				VCOMPtr(const VCOMPtr& p)		{ fPtr = p.fPtr;					if ( fPtr ) { fPtr->AddRef(); } }
				~VCOMPtr()						{ this->Release(); }

	VCOMError	Query(const VWIID& iid)			{ 
													this->Release();
													IVWUnknown*	pInterface		= NULL;
													VCOMError err = ::GS_VWQueryInterface( iid, & pInterface );
													if ( VCOM_SUCCEEDED( err ) ) {
														// Vlado: Temporally removed 'dynamic_cast'
														// because on new Mac OS it is failing to cast properly
														//fPtr = dynamic_cast<T*>( pInterface );
														fPtr = (T*)( pInterface );
														if ( fPtr == NULL ) {
															if ( pInterface ) {
																pInterface->Release();
																pInterface	= NULL;
															}
															// the passed 'iid' is not from the extected type 'T'
															err			= kVCOMError_InvalidArg;
														}
													}
													return err;
												}
	void		AddRef()						{ if ( fPtr ) { fPtr->AddRef(); } }
	long		Release()						{ long res = 0; if ( fPtr ) { res = fPtr->Release(); } fPtr = NULL; return res; }

	VCOMPtr&	operator=(T* ptr)				{ T* pOld = fPtr; fPtr = ptr;						if ( fPtr ) { fPtr->AddRef(); } if ( pOld ) { pOld->Release(); } return *this; }
	VCOMPtr&	operator=(const VCOMPtr& p)		{ T* pOld = fPtr; fPtr = p.fPtr;					if ( fPtr ) { fPtr->AddRef(); } if ( pOld ) { pOld->Release(); } return *this; }

	T*			operator->() const				{ return fPtr; }
	T**			operator&()						{ return & fPtr; }
				operator T*() const 			{ return fPtr; }

protected:
	T*			Get() const	{ return fPtr; }
	void		Set(T* p)	{ fPtr = p; }

private:
	T*		fPtr;
};

typedef VCOMPtr<IVWUnknown>		IVWUnknownPtr;

// VCOM interface implementation with late destruction
template<class Interface> class VCOMImpl : public Interface
{
public:
				VCOMImpl()						{ fRefCnt = 0; fParent = NULL; }
				VCOMImpl(IVWUnknown* parent)	{ fRefCnt = 0; fParent = parent; if ( fParent ) { fParent->AddRef(); } }
	virtual		~VCOMImpl()						{ }

// IVWUnknown
public:
	virtual long VCOM_CALLTYPE		AddRef()	{ fRefCnt++; return fRefCnt; }
	virtual long VCOM_CALLTYPE		Release()
									{
										ASSERTN( kEveryone, fRefCnt > 0 );
										if ( fRefCnt > 0 ) {
											fRefCnt --;
										}
										if ( fRefCnt == 0 ) {
											this->OnRefCountZero();
											// exit immediatelly. 'this' may no longer be valid
											return 0;
										}
										return fRefCnt;
									}

protected:
	// notification when this instance ref count goes down to zero
	virtual void					OnRefCountZero() { if ( fParent ) { fParent->Release(); } fParent = NULL; }

protected:
	long		fRefCnt;
	IVWUnknown* fParent;
};

// VCOM interface implementation with immediate destruction
template<class Interface> class VCOMImmediateImpl : public Interface
{
public:
				VCOMImmediateImpl()						{ fRefCnt = 0; fParent = NULL; }
				VCOMImmediateImpl(IVWUnknown* parent)	{ fRefCnt = 0; fParent = parent; if ( fParent ) { fParent->AddRef(); } }
	virtual		~VCOMImmediateImpl()					{ }

// IVWUnknown
public:
	virtual long VCOM_CALLTYPE		AddRef()	{ fRefCnt++; return fRefCnt; }
	virtual long VCOM_CALLTYPE		Release()
									{
										ASSERTN( kEveryone, fRefCnt > 0 );
										if ( fRefCnt > 0 ) {
											fRefCnt --;

											// mechanizm for immediate delete of the interface instance
											if ( fRefCnt == 0 ) {
												this->OnRefCountZero();
												::GS_VWNotifyDeleteInterface( this );
												// EXIT IMMEDIATELY! 'this' no longer exist!!!
												return 0;
											}
										}
										return fRefCnt;
									}

protected:
	// notification when this instance ref count goes down to zero
	virtual void					OnRefCountZero() { if ( fParent ) { fParent->Release(); } fParent = NULL; }

protected:
	long		fRefCnt;
	IVWUnknown* fParent;
};


//
///
/////
///////
/////////       ---------------------------------------------------------------
///////////		END PLUGIN LIBRARY LIBRARY VCOM SUPPORT -------------------------------------------------
/////////////	---------------------------------------------------------------

/////////////	---------------------------------------------------------------
// Parametric plug type accessed via ovParametricInternalID


typedef short		TInternalID;


// This is the list of plugin Internal ID constants which are mapped to the matching names below
// If you are adding a plugin object, add it to the bottom of the list and make sure the number increments by 1.
const TInternalID kInternalID_AcornNutInch				=	1;
const TInternalID kInternalID_AcornNutInch3D			=	2;
const TInternalID kInternalID_AdjacencyMatrix			=	3;
const TInternalID kInternalID_AdjacencyScore			=	4;
const TInternalID kInternalID_Angle						=	5;
const TInternalID kInternalID_AngleInch					=	6;
const TInternalID kInternalID_AngleMetric				=	7;
const TInternalID kInternalID_Angle3D					=	8;
const TInternalID kInternalID_AreaPlant					=	9;
const TInternalID kInternalID_BallBearing				=	10;
const TInternalID kInternalID_BallBearing3D				=	11;
const TInternalID kInternalID_BaseCabinet				=	12;
const TInternalID kInternalID_BathShower				=	13;
const TInternalID kInternalID_BattInsulation			=	14;
const TInternalID kInternalID_BearingLockNut			=	15;
const TInternalID kInternalID_BearingLockNut3D			=	16;
const TInternalID kInternalID_BellevilleSpring			=	17;
const TInternalID kInternalID_BellevilleSpring3D		=	18;
const TInternalID kInternalID_BevelGears				=	19;
const TInternalID kInternalID_BevelGears3D				=	20;
const TInternalID kInternalID_BlendedProjector			=	21;
const TInternalID kInternalID_BlendedScreen				=	22;
const TInternalID kInternalID_Board						=	23;
const TInternalID kInternalID_BoltandNutInch			=	24;
const TInternalID kInternalID_BoltandNutmm				=	25;
const TInternalID kInternalID_BoxObject					=	26;
const TInternalID kInternalID_BreakLine					=	27;
const TInternalID kInternalID_Bridgeline				=	28;
const TInternalID kInternalID_BulbFlat					=	29;
const TInternalID kInternalID_BulbFlat3D				=	30;
const TInternalID kInternalID_Callout					=	31;
const TInternalID kInternalID_Cam						=	32;
const TInternalID kInternalID_Campanile					=	33;
const TInternalID kInternalID_CapScrewDIN				=	34;
const TInternalID kInternalID_CapScrewDIN3D				=	35;
const TInternalID kInternalID_CapScrewInch				=	36;
const TInternalID kInternalID_CapScrewInch3D			=	37;
const TInternalID kInternalID_CapScrewISO				=	38;
const TInternalID kInternalID_CapScrewISO3D				=	39;
const TInternalID kInternalID_CapScrewMetric			=	40;
const TInternalID kInternalID_CapScrewMetric3D			=	41;
const TInternalID kInternalID_CarriageBolt				=	42;
const TInternalID kInternalID_CarriageBolt3D			=	43;
const TInternalID kInternalID_CeilingGrid				=	44;
const TInternalID kInternalID_CenterDrill				=	45;
const TInternalID kInternalID_CenterDrill3D				=	46;
const TInternalID kInternalID_CenterLineMarker			=	47;
const TInternalID kInternalID_ChainExtrude				=	48;
const TInternalID kInternalID_Channel					=	49;
const TInternalID kInternalID_ChannelInch				=	50;
const TInternalID kInternalID_ChannelMetric				=	51;
const TInternalID kInternalID_Channel3D					=	52;
const TInternalID kInternalID_CircularStair				=	53;
const TInternalID kInternalID_ClevisPinDIN				=	54;
const TInternalID kInternalID_ClevisPinDIN3D			=	55;
const TInternalID kInternalID_ClevisPinInch				=	56;
const TInternalID kInternalID_ClevisPinInch3D			=	57;
const TInternalID kInternalID_ClevisPinMetric			=	58;
const TInternalID kInternalID_ClevisPinMetric3D			=	59;
const TInternalID kInternalID_ClothesRod				=	60;
const TInternalID kInternalID_Column2					=	61;
const TInternalID kInternalID_CommDevice				=	62;
const TInternalID kInternalID_CompartmentSink			=	63;
const TInternalID kInternalID_CompressionSpring1		=	64;
const TInternalID kInternalID_CompressionSpring13D		=	65;
const TInternalID kInternalID_CompressionSpring2		=	66;
const TInternalID kInternalID_CompressionSpring23D		=	67;
const TInternalID kInternalID_ConeObject				=	68;
const TInternalID kInternalID_ConicalCompSpring3D		=	69;
const TInternalID kInternalID_ConicalCompressionSpring	=	70;
const TInternalID kInternalID_Coordinate				=	71;
const TInternalID kInternalID_CotterPinInch				=	72;
const TInternalID kInternalID_CotterPinInch3D			=	73;
const TInternalID kInternalID_CounterTop				=	74;
const TInternalID kInternalID_CurtainObject				=	75;
const TInternalID kInternalID_CurvedTruss				=	76;
const TInternalID kInternalID_CustomStair				=	77;
const TInternalID kInternalID_CuttingPlane				=	78;
const TInternalID kInternalID_CuttingPlane1				=	79;
const TInternalID kInternalID_CuttingPlane2				=	80;
const TInternalID kInternalID_CylinderObject			=	81;
const TInternalID kInternalID_DataStamp					=	82;
const TInternalID kInternalID_DatumFeatureSymbol		=	83;
const TInternalID kInternalID_DatumTargetSymbol			=	84;
const TInternalID kInternalID_Desk						=	85;
const TInternalID kInternalID_DetailBubble				=	86;
const TInternalID kInternalID_DetailCutWood				=	87;
const TInternalID kInternalID_DialogBuilderLink			=	88;
const TInternalID kInternalID_DialogBuilderControl		=	89;
const TInternalID kInternalID_DieSpring					=	90;
const TInternalID kInternalID_DieSpring3D				=	91;
const TInternalID kInternalID_Door						=	92;
const TInternalID kInternalID_DowelPinInch				=	93;
const TInternalID kInternalID_DowelPinInch3D			=	94;
const TInternalID kInternalID_DrawingBorderUniversal	=	95;
const TInternalID kInternalID_DrawingLabel				=	96;
const TInternalID kInternalID_DrilledFooting			=	97;
const TInternalID kInternalID_DripEmitter				=	98;
const TInternalID kInternalID_DTM6						=	99;
const TInternalID kInternalID_DTM6Snapshot				=	100;
const TInternalID kInternalID_DwgLabelSimple			=	101;
const TInternalID kInternalID_ElevationBenchmark		=	102;
const TInternalID kInternalID_EllipsoidObject			=	103;
const TInternalID kInternalID_Escalator					=	104;
const TInternalID kInternalID_ExtensionSpring3D			=	105;
const TInternalID kInternalID_ExtensionSpringEnd		=	106;
const TInternalID kInternalID_ExtensionSpringFront		=	107;
const TInternalID kInternalID_ExtrudeAlongPath			=	108;
const TInternalID kInternalID_EyeBolt					=	109;
const TInternalID kInternalID_EyeBolt3D					=	110;
const TInternalID kInternalID_FeatureControlFrame		=	111;
const TInternalID kInternalID_Fireplace					=	112;
const TInternalID kInternalID_FlangedBearing2Hole		=	113;
const TInternalID kInternalID_FlangedBearing2Hole3D		=	114;
const TInternalID kInternalID_FlangedBearing4Hole		=	115;
const TInternalID kInternalID_FlangedBearing4Hole3D		=	116;
const TInternalID kInternalID_FlowchartLink				=	117;
const TInternalID kInternalID_FlowchartNode				=	118;
const TInternalID kInternalID_FocusPointObject			=	119;
const TInternalID kInternalID_FramingMember				=	120;
const TInternalID kInternalID_GeneralNotes				=	121;
const TInternalID kInternalID_GenevaMechanism			=	122;
const TInternalID kInternalID_GeomDimandTolNote			=	123;
const TInternalID kInternalID_GrabBars					=	124;
const TInternalID kInternalID_GridPolar					=	125;
const TInternalID kInternalID_GridRectangular			=	126;
const TInternalID kInternalID_GridBubble				=	127;
const TInternalID kInternalID_GuardrailCurved			=	128;
const TInternalID kInternalID_GuardrailStraight			=	129;
const TInternalID kInternalID_HandrailCurved			=	130;
const TInternalID kInternalID_HandrailStraight			=	131;
const TInternalID kInternalID_Hardscape					=	132;
const TInternalID kInternalID_HardscapeVS				=	133;
const TInternalID kInternalID_HelixSpiral				=	134;
const TInternalID kInternalID_HexBoltDIN				=	135;
const TInternalID kInternalID_HexBoltDIN3D				=	136;
const TInternalID kInternalID_HexBoltInch				=	137;
const TInternalID kInternalID_HexBoltInch3D				=	138;
const TInternalID kInternalID_HexBoltISO				=	139;
const TInternalID kInternalID_HexBoltISO3D				=	140;
const TInternalID kInternalID_HexBoltMetric				=	141;
const TInternalID kInternalID_HexBoltMetric3D			=	142;
const TInternalID kInternalID_HexBoltFineDIN			=	143;
const TInternalID kInternalID_HexBoltFineDIN3D			=	144;
const TInternalID kInternalID_HexBoltFlangeHeadDIN		=	145;
const TInternalID kInternalID_HexBoltFlgeHeadDIN3D		=	146;
const TInternalID kInternalID_HexCapNutDIN				=	147;
const TInternalID kInternalID_HexCapNutDIN3D			=	148;
const TInternalID kInternalID_HoleDrilled				=	149;
const TInternalID kInternalID_HoleDrilled3D				=	150;
const TInternalID kInternalID_HoleTappedInch			=	151;
const TInternalID kInternalID_HoleTappedInch3D			=	152;
const TInternalID kInternalID_HoleTappedMetric			=	153;
const TInternalID kInternalID_HoleTappedMetric3D		=	154;
const TInternalID kInternalID_HolePattern				=	155;
const TInternalID kInternalID_Hub3D						=	156;
const TInternalID kInternalID_HumanFigure				=	157;
const TInternalID kInternalID_HVACDamper				=	158;
const TInternalID kInternalID_HVACDiffuser				=	159;
const TInternalID kInternalID_HVACElbowDuct				=	160;
const TInternalID kInternalID_HVACFlexDuct				=	161;
const TInternalID kInternalID_HVACOutlet				=	162;
const TInternalID kInternalID_HVACSplitter				=	163;
const TInternalID kInternalID_HVACStraightDuct			=	164;
const TInternalID kInternalID_HVACTransition			=	165;
const TInternalID kInternalID_HVACVerticalDuct			=	166;
const TInternalID kInternalID_HVACVerticalElbow			=	167;
const TInternalID kInternalID_IBeam						=	168;
const TInternalID kInternalID_IBeamInch					=	169;
const TInternalID kInternalID_IBeamMetric				=	170;
const TInternalID kInternalID_IBeam3D					=	171;
const TInternalID kInternalID_IDLeader					=	172;
const TInternalID kInternalID_IFCEntity					=	173;
const TInternalID kInternalID_ImageProp					=	174;
const TInternalID kInternalID_IncandescentFixture		=	175;
const TInternalID kInternalID_IrrigationHead			=	176;
const TInternalID kInternalID_IrrigationLine			=	177;
const TInternalID kInternalID_JBolt						=	178;
const TInternalID kInternalID_JBolt3D					=	179;
const TInternalID kInternalID_Joist						=	180;
const TInternalID kInternalID_Key						=	181;
const TInternalID kInternalID_Key3D						=	182;
const TInternalID kInternalID_KeynoteLegend				=	183;
const TInternalID kInternalID_Keyway					=	184;
const TInternalID kInternalID_KnurledThumbNutDIN		=	185;
const TInternalID kInternalID_KnurledThumbNutDIN3D		=	186;
const TInternalID kInternalID_KnurledThumbNutInch		=	187;
const TInternalID kInternalID_KnurledThumbNutInch3D		=	188;
const TInternalID kInternalID_LagScrew					=	189;
const TInternalID kInternalID_LagScrew3D				=	190;
const TInternalID kInternalID_LandscapeArea				=	191;
const TInternalID kInternalID_LandscapeWall				=	192;
const TInternalID kInternalID_LandscapeWallArc			=	193;
const TInternalID kInternalID_LandscapeWallBez			=	194;
const TInternalID kInternalID_LeaderLine				=	195;
const TInternalID kInternalID_LightPositionObj			=	196;
const TInternalID kInternalID_LightingDevice			=	197;
const TInternalID kInternalID_LinearMaterial			=	198;
const TInternalID kInternalID_LinkedMarker				=	199;
const TInternalID kInternalID_LockWasherDIN				=	200;
const TInternalID kInternalID_LockWasherDIN3D			=	201;
const TInternalID kInternalID_LockWasherInch			=	202;
const TInternalID kInternalID_LockWasherInch3D			=	203;
const TInternalID kInternalID_LockWasherISO				=	204;
const TInternalID kInternalID_LockWasherISO3D			=	205;
const TInternalID kInternalID_LockWasherMetric			=	206;
const TInternalID kInternalID_LockWasherMetric3D		=	207;
const TInternalID kInternalID_MachineScrewDIN			=	208;
const TInternalID kInternalID_MachineScrewDIN3D			=	209;
const TInternalID kInternalID_MachineScrewInch			=	210;
const TInternalID kInternalID_MachineScrewInch3D		=	211;
const TInternalID kInternalID_MachineScrewISO			=	212;
const TInternalID kInternalID_MachineScrewISO3D			=	213;
const TInternalID kInternalID_MachineScrewMetric		=	214;
const TInternalID kInternalID_MachineScrewMetric3D		=	215;
const TInternalID kInternalID_MassingModel				=	216;
const TInternalID kInternalID_MaterialNote				=	217;
const TInternalID kInternalID_Mullion					=	218;
const TInternalID kInternalID_NeedleBearing				=	219;
const TInternalID kInternalID_NeedleBearing3D			=	220;
const TInternalID kInternalID_NNA_ArchSection			=	221;
const TInternalID kInternalID_NNA_ChainDim				=	222;
const TInternalID kInternalID_NNA_DesignLayerViewport	=	223;
const TInternalID kInternalID_NorthArrow				=	224;
const TInternalID kInternalID_NutDIN					=	225;
const TInternalID kInternalID_NutDIN3D					=	226;
const TInternalID kInternalID_NutInch					=	227;
const TInternalID kInternalID_NutInch3D					=	228;
const TInternalID kInternalID_NutISO					=	229;
const TInternalID kInternalID_NutISO3D					=	230;
const TInternalID kInternalID_NutMetric					=	231;
const TInternalID kInternalID_NutMetric3D				=	232;
const TInternalID kInternalID_ParabaloidObject			=	233;
const TInternalID kInternalID_ParallelPinDIN			=	234;
const TInternalID kInternalID_ParallelPinDIN3D			=	235;
const TInternalID kInternalID_ParkingAlongPath			=	236;
const TInternalID kInternalID_ParkingArea				=	237;
const TInternalID kInternalID_ParkingSpaces				=	238;
const TInternalID kInternalID_PartsList					=	239;
const TInternalID kInternalID_PhotoGrid					=	240;
const TInternalID kInternalID_Photometer				=	241;
const TInternalID kInternalID_Pilaster2					=	242;
const TInternalID kInternalID_PillowBlock				=	243;
const TInternalID kInternalID_PillowBlock3D				=	244;
const TInternalID kInternalID_PipeFitting				=	245;
const TInternalID kInternalID_Piping					=	246;
const TInternalID kInternalID_PipingConnection			=	247;
const TInternalID kInternalID_PipingRun					=	248;
const TInternalID kInternalID_PlainWasherDIN			=	249;
const TInternalID kInternalID_PlainWasherDIN3D			=	250;
const TInternalID kInternalID_PlainWasherInch			=	251;
const TInternalID kInternalID_PlainWasherInch3D			=	252;
const TInternalID kInternalID_PlainWasherISO			=	253;
const TInternalID kInternalID_PlainWasherISO3D			=	254;
const TInternalID kInternalID_PlainWasherMetric			=	255;
const TInternalID kInternalID_PlainWasherMetric3D		=	256;
const TInternalID kInternalID_Plant						=	257;
const TInternalID kInternalID_PropertyLine				=	258;
const TInternalID kInternalID_Pulley					=	259;
const TInternalID kInternalID_Pulley3D					=	260;
const TInternalID kInternalID_PyramidObject				=	261;
const TInternalID kInternalID_QuarterArc				=	262;
const TInternalID kInternalID_Rafter					=	263;
const TInternalID kInternalID_Ramp						=	264;
const TInternalID kInternalID_Receptacle				=	265;
const TInternalID kInternalID_RectangularTubing			=	266;
const TInternalID kInternalID_RectangularTubingInch		=	267;
const TInternalID kInternalID_RectangularTubingMetric	=	268;
const TInternalID kInternalID_RectangularTubing3D		=	269;
const TInternalID kInternalID_RedlineObject				=	270;
const TInternalID kInternalID_ReferenceMarker			=	271;
const TInternalID kInternalID_RepetitiveUnit			=	272;
const TInternalID kInternalID_RetainingRingASME			=	273;
const TInternalID kInternalID_RetainingRingASME3D		=	274;
const TInternalID kInternalID_RetainingRingDIN			=	275;
const TInternalID kInternalID_RetainingRingDIN3D		=	276;
const TInternalID kInternalID_RetainingWasherDIN		=	277;
const TInternalID kInternalID_RetainingWasherDIN3D		=	278;
const TInternalID kInternalID_RevisionBubble			=	279;
const TInternalID kInternalID_RevisionCloud				=	280;
const TInternalID kInternalID_RevisionMarker			=	281;
const TInternalID kInternalID_RivetDIN					=	282;
const TInternalID kInternalID_RivetDIN3D				=	283;
const TInternalID kInternalID_RivetLargeInch			=	284;
const TInternalID kInternalID_RivetLargeInch3D			=	285;
const TInternalID kInternalID_RivetSmallInch			=	286;
const TInternalID kInternalID_RivetSmallInch3D			=	287;
const TInternalID kInternalID_RoadwayBezier				=	288;
const TInternalID kInternalID_RoadwayCurved				=	289;
const TInternalID kInternalID_RoadwayNURBS				=	290;
const TInternalID kInternalID_RoadwayStraight			=	291;
const TInternalID kInternalID_RoadwayTee				=	292;
const TInternalID kInternalID_RollerBearing				=	293;
const TInternalID kInternalID_RollerBearing3D			=	294;
const TInternalID kInternalID_RollerChainCircular		=	295;
const TInternalID kInternalID_RollerChainCircular3D		=	296;
const TInternalID kInternalID_RollerChainLinear			=	297;
const TInternalID kInternalID_RollerChainLinear3D		=	298;
const TInternalID kInternalID_RollerChainOffsetLink		=	299;
const TInternalID kInternalID_RollerChainContinuous		=	300;
const TInternalID kInternalID_RollerChainContinuous3D	=	301;
const TInternalID kInternalID_RollerChainOffsetLink3D	=	302;
const TInternalID kInternalID_RoomName					=	303;
const TInternalID kInternalID_RoomNameSimple			=	304;
const TInternalID kInternalID_RoundTubing				=	305;
const TInternalID kInternalID_RoundTubingInch			=	306;
const TInternalID kInternalID_RoundTubingMetric			=	307;
const TInternalID kInternalID_RoundTubing3D				=	308;
const TInternalID kInternalID_ScaleBar					=	309;
const TInternalID kInternalID_ScrewandNutDIN			=	310;
const TInternalID kInternalID_ScrewandNutDIN3D			=	311;
const TInternalID kInternalID_ScrewandNutInch			=	312;
const TInternalID kInternalID_ScrewandNutInch3D			=	313;
const TInternalID kInternalID_ScrewandNutISO			=	314;
const TInternalID kInternalID_ScrewandNutISO3D			=	315;
const TInternalID kInternalID_ScrewandNutMetric			=	316;
const TInternalID kInternalID_ScrewandNutMetric3D		=	317;
const TInternalID kInternalID_ScrewThreads				=	318;
const TInternalID kInternalID_SeatingLayout				=	319;
const TInternalID kInternalID_SectionNote				=	320;
const TInternalID kInternalID_SectionElevationMarker	=	321;
const TInternalID kInternalID_SetScrewDIN				=	322;
const TInternalID kInternalID_SetScrewDIN3D				=	323;
const TInternalID kInternalID_SetScrewInch				=	324;
const TInternalID kInternalID_SetScrewInch3D			=	325;
const TInternalID kInternalID_SetScrewISO				=	326;
const TInternalID kInternalID_SetScrewISO3D				=	327;
const TInternalID kInternalID_SetScrewMetric			=	328;
const TInternalID kInternalID_SetScrewMetric3D			=	329;
const TInternalID kInternalID_Shaft						=	330;
const TInternalID kInternalID_Shaft3D					=	331;
const TInternalID kInternalID_ShaftBreak				=	332;
const TInternalID kInternalID_ShaftBreak2				=	333;
const TInternalID kInternalID_ShaftSegmentStraight		=	334;
const TInternalID kInternalID_ShaftSegmentStraight3D	=	335;
const TInternalID kInternalID_ShaftSegmentTapered		=	336;
const TInternalID kInternalID_ShaftSegmentTapered3D		=	337;
const TInternalID kInternalID_SheetMetalScrew			=	338;
const TInternalID kInternalID_SheetMetalScrew3D			=	339;
const TInternalID kInternalID_ShelvingUnit				=	340;
const TInternalID kInternalID_ShoulderScrewDIN			=	341;
const TInternalID kInternalID_ShoulderScrewDIN3D		=	342;
const TInternalID kInternalID_ShoulderScrewInch			=	343;
const TInternalID kInternalID_ShoulderScrewInch3D		=	344;
const TInternalID kInternalID_ShoulderScrewISO			=	345;
const TInternalID kInternalID_ShoulderScrewISO3D		=	346;
const TInternalID kInternalID_ShoulderScrewMetric		=	347;
const TInternalID kInternalID_ShoulderScrewMetric3D		=	348;
const TInternalID kInternalID_SimpleElevator			=	349;
const TInternalID kInternalID_SimpleStair				=	350;
const TInternalID kInternalID_SiteModifiers				=	351;
const TInternalID kInternalID_SlopeDimension			=	352;
const TInternalID kInternalID_Slot						=	353;
const TInternalID kInternalID_Space						=	354;
const TInternalID kInternalID_SpaceLink					=	355;
const TInternalID kInternalID_SphereObject				=	356;
const TInternalID kInternalID_Spiral					=	357;
const TInternalID kInternalID_Sprocket					=	358;
const TInternalID kInternalID_Sprocket3D				=	359;
const TInternalID kInternalID_SpurGear					=	360;
const TInternalID kInternalID_SpurGearRack				=	361;
const TInternalID kInternalID_SpurGearRack3D			=	362;
const TInternalID kInternalID_SpurGear3D				=	363;
const TInternalID kInternalID_SquareTubing				=	364;
const TInternalID kInternalID_SquareTubingInch			=	365;
const TInternalID kInternalID_SquareTubingMetric		=	366;
const TInternalID kInternalID_SquareTubing3D			=	367;
const TInternalID kInternalID_StackingDiagram			=	368;
const TInternalID kInternalID_Stage						=	369;
const TInternalID kInternalID_StakeObject				=	370;
const TInternalID kInternalID_Stipple					=	371;
const TInternalID kInternalID_Storypole					=	372;
const TInternalID kInternalID_StraightTruss				=	373;
const TInternalID kInternalID_SurfaceTextureSymbol		=	374;
const TInternalID kInternalID_SwingBolt					=	375;
const TInternalID kInternalID_SwingBolt3D				=	376;
const TInternalID kInternalID_SwingEyeBolt				=	377;
const TInternalID kInternalID_SwingEyeBolt3D			=	378;
const TInternalID kInternalID_Switch					=	379;
const TInternalID kInternalID_TBolt						=	380;
const TInternalID kInternalID_TBolt3D					=	381;
const TInternalID kInternalID_Table						=	382;
const TInternalID kInternalID_TableandChairs			=	383;
const TInternalID kInternalID_TaperPinDIN				=	384;
const TInternalID kInternalID_TaperPinDIN3D				=	385;
const TInternalID kInternalID_TaperPinInch				=	386;
const TInternalID kInternalID_TaperPinInch3D			=	387;
const TInternalID kInternalID_TaperedExtrude			=	388;
const TInternalID kInternalID_TaperedRollerBearing		=	389;
const TInternalID kInternalID_TaperedRollerBearing3D	=	390;
const TInternalID kInternalID_Tee						=	391;
const TInternalID kInternalID_TeeInch					=	392;
const TInternalID kInternalID_TeeMetric					=	393;
const TInternalID kInternalID_Tee3D						=	394;
const TInternalID kInternalID_Television				=	395;
const TInternalID kInternalID_TextAlongPath				=	396;
const TInternalID kInternalID_ThreadedShaft				=	397;
const TInternalID kInternalID_ThreadedShaft3D			=	398;
const TInternalID kInternalID_ThrustBearing				=	399;
const TInternalID kInternalID_ThrustBearing3D			=	400;
const TInternalID kInternalID_ThumbScrewInch			=	401;
const TInternalID kInternalID_ThumbScrewInch3D			=	402;
const TInternalID kInternalID_ToiletStall				=	403;
const TInternalID kInternalID_TorsionSpring3D			=	404;
const TInternalID kInternalID_TorsionSpringEnd			=	405;
const TInternalID kInternalID_TorsionSpringFront		=	406;
const TInternalID kInternalID_TorusObject				=	407;
const TInternalID kInternalID_TubularRivetDIN			=	408;
const TInternalID kInternalID_TubularRivetDIN3D			=	409;
const TInternalID kInternalID_TwoFerObject				=	410;
const TInternalID kInternalID_UBolt						=	411;
const TInternalID kInternalID_UBolt3D					=	412;
const TInternalID kInternalID_UtilityCabinet			=	413;
const TInternalID kInternalID_VideoScreen				=	414;
const TInternalID kInternalID_WallCabinet				=	415;
const TInternalID kInternalID_WallFeature				=	416;
const TInternalID kInternalID_WeldingSymFilletAWS		=	417;
const TInternalID kInternalID_WeldingSymFilletISO		=	418;
const TInternalID kInternalID_WeldingSymFlangeAWS		=	419;
const TInternalID kInternalID_WeldingSymFlangeISO		=	420;
const TInternalID kInternalID_WeldingSymGrooveAWS		=	421;
const TInternalID kInternalID_WeldingSymGrooveISO		=	422;
const TInternalID kInternalID_WeldingSymMiscAWS			=	423;
const TInternalID kInternalID_WeldingSymMiscISO			=	424;
const TInternalID kInternalID_WeldingSymSlotPlugAWS		=	425;
const TInternalID kInternalID_WeldingSymSlotPlugISO		=	426;
const TInternalID kInternalID_WideFlange				=	427;
const TInternalID kInternalID_WideFlangeInch			=	428;
const TInternalID kInternalID_WideFlangeMetric			=	429;
const TInternalID kInternalID_WideFlange3D				=	430;
const TInternalID kInternalID_Window					=	431;
const TInternalID kInternalID_WindowWallCurved			=	432;
const TInternalID kInternalID_WindowWallStraight		=	433;
const TInternalID kInternalID_WingNutDIN				=	434;
const TInternalID kInternalID_WingNutDIN3D				=	435;
const TInternalID kInternalID_WingNutTypeAInch			=	436;
const TInternalID kInternalID_WingNutTypeAInch3D		=	437;
const TInternalID kInternalID_WingNutTypeBInch			=	438;
const TInternalID kInternalID_WingNutTypeBInch3D		=	439;
const TInternalID kInternalID_WingNutTypeCInch			=	440;
const TInternalID kInternalID_WingNutTypeCInch3D		=	441;
const TInternalID kInternalID_WingNutTypeDInch			=	442;
const TInternalID kInternalID_WingNutTypeDInch3D		=	443;
const TInternalID kInternalID_WoodScrew					=	444;
const TInternalID kInternalID_WoodScrew3D				=	445;
const TInternalID kInternalID_WoodruffKey				=	446;
const TInternalID kInternalID_WoodruffKey3D				=	447;
const TInternalID kInternalID_WorkstationCounter		=	448;
const TInternalID kInternalID_WorkstationOverhead		=	449;
const TInternalID kInternalID_WorkstationPanel			=	450;
const TInternalID kInternalID_WorkstationPedestal		=	451;
const TInternalID kInternalID_Worm						=	452;
const TInternalID kInternalID_Worm3D					=	453;
const TInternalID kInternalID_WormGear					=	454;
const TInternalID kInternalID_WormGear3D				=	455;
const TInternalID kInternalID_ZSection					=	456;
const TInternalID kInternalID_ZSection3D				=	457;
const TInternalID kInternalID___PDF						=	458;
const TInternalID kInternalID_Slab						=	459;
const TInternalID kInternalID_RenderWorks_Camera		=	460;

/////////////	---------------------------------------------------------------
/////////////	---------------------------------------------------------------
/////////////	---------------------------------------------------------------
/////////////	---------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif
	typedef void GS_API_PTR(GenericFunctionPtr)(void);
	typedef GenericFunctionPtr GS_API_PTR(RetrieverPtr)(long id);
#ifdef __cplusplus
}
#endif

	typedef RetrieverPtr RetrieverProcUPP;

	#define CallRetrieverProc(userRoutine, id)		\
			(*(userRoutine))((id))
	#define NewRetrieverProc(userRoutine)		\
			(RetrieverProcUPP) (userRoutine)

#ifdef _MINICAD_
	extern "C" GenericFunctionPtr GS_API FIDToPtr(long id);
	extern RetrieverProcUPP gFIDToPtrUPP;
#endif

	struct GS_API CallBackBlock {
		RetrieverProcUPP	rPtr;
		#ifndef _WINDOWS
		long				a4store;	// *** JAK *** We dont need this. Remove it soon.
		#endif
		
		#ifdef _MINICAD_
		CallBackBlock()
			{ rPtr = gFIDToPtrUPP; }
		#endif
		CallBackBlock(const CallBackBlock *cbp)
			{ this->rPtr = cbp ? cbp->rPtr : NULL; }
	};
	typedef CallBackBlock *CallBackPtr;


////////////////////////
// GLOBAL CallBackPtr
//   In VectorWorks, this should be passed to all GS_ calls and does nothing.
//   In externals, this should be used as the global callback pointer.
////////////////////////

extern CallBackPtr gCBP;

////////////////////////
// GLOBAL pointer to the SDK VCOM funtion interface
//   In VectorWorks, this is initialized during start up
//   In externals, this should be queried in the main.
////////////////////////
namespace VectorWorks { class ISDK; }

extern VectorWorks::ISDK*	gSDK;

////////////////////////
// GSCharsHandle
//   Used in VectorWorks objects which maintain a handle to characters
////////////////////////

typedef char **GSCharsHandle;


////////////////////////
// MCObjectHandle
//   In VectorWorks, this allows access to BlankHandle
//   In externals, this provides opaque access
//   The VectorWorks internal version is in InitGlobals.h
////////////////////////

#ifndef _MINICAD_
	typedef GSHandle MCObjectHandle;
#endif


// MenuChunkInfo structure
#ifdef __cplusplus
struct MenuChunkInfo {			// used by menu definition functions
	short menuID;
	short itemID;
	short modifiers;
	union {
		short chunkIndex;		// used by kMenuDoInterface and kMenuItemDisplayContextualHelp
		short chunkSize;		// used by kMenuCheckHilite

		#ifdef GS_INTERFACE_FACTORING
		short commandID;        // used by kMenuAddItems
		#endif

	};
};

typedef MenuChunkInfo *MenuChunkInfoPtr;


/////////////////////////////
// MenuItemSpec is used by kMenuItemCheck action
//
// 0 is returned to allow execution of item
// kMenuItemSecurity is returned to disallow execution

struct MenuItemSpec {					
	short			commandID;			// if commandID < 0 then it's internal & externalName is nil
	GSStringPtr		externalName;		// if commandID == 1 then it's plug-in binary & externalName points to the name (no extension)
										// if commandID == 2 then it's plug-in VectorScript & externalName points to the name (no extension)
};

/////////////////////////////
//	ToolActionHandlerParams is a wrapper for tool event data.
//
class ToolActionHandlerParams
{
public:
	ToolActionHandlerParams(long action, long message1, long message2) :
		fToolAction(action),
		fToolMessage1(message1),
		fToolMessage2(message2)
	{}
	
	long fToolAction;
	long fToolMessage1;
	long fToolMessage2;
};

/////////////////////////////
// MenuToolActionHandlerParams is used by kMenuHandleToolAction menu action.
//
//	If a Menu wants to get points from the document it needs to return kMenuRunTempTool
//	from its kMenuDoInterface Menu action handler.  The menu will then be called with 
//	kMenuHandleToolActions actions with message1 containing a pointer to a MenuToolActionHandlerParams
//	structure.  MenuToolActionHandlerParams contains data defining standard tool actions.
//	The menu command must handle these tool actions as if it were a tool definition procedure.
//	See tool action handling above for more.
//
class MenuToolActionHandlerParams : public ToolActionHandlerParams
//
//	In kMenuHandleToolActions action calls the message1 param contains a pointer to this structure.  Use
//	fToolAction, fToolMessage1, and fToolMessage2
{
public:
	MenuToolActionHandlerParams(long action, long message1, long message2) :
		ToolActionHandlerParams(action, message1, message2)
	{}
};

// *PCP* - this may need to go back in TDQuickdraw.h
typedef long (*MenuItemDefProcPtr)(long action, long message, long& userData, CallBackPtr defData);

//-----------------------------------------------------------------
typedef MenuItemDefProcPtr MenuItemDefUPP;

#define CallMenuItemDefProc(userRoutine, action, message, userData, defData)		\
		(*(userRoutine))((action), (message), (userData), (defData))
#define New68KMenuItemDefProc(userRoutine)		\
		(MenuItemDefUPP) (userRoutine)

#endif

//-----------------------------------------------------------------
// GS_SDKPPC_JAK
// GS_SDKPPC_JAK2

////typedef ToolDefProcPtr GSPluginToolUPP;	// **************************
////#define CallGSPluginToolProc
////#define NewGSPluginToolProc



#ifndef GSEBUILDTYPES
	#define GSEBUILDTYPES
	const short kMiniCad = 1;
	const short kBlueprint = 2;
#endif

#if defined(_WINDOWS) && !defined(_MINICAD_)
	#if !defined(UNDEF_PASCAL) || (UNDEF_PASCAL != 0)
		#ifdef pascal
			#undef pascal
		#endif
		#define pascal
	#endif
#endif



// these are the most commonly used list size constants
// See LDef.h for additional size constants
#if _WINDOWS

	const short kSmallListElementHeight=	14;
	const short kLargeListElementHeight=	14;
	const short kIconListElementHeight=		32;
	const short kSICNListElementHeight=		16;
	
#else

	const short kSmallListElementHeight=	12;
	const short kLargeListElementHeight=	16;
	const short kIconListElementHeight=		32;
	const short kSICNListElementHeight=		16;

#endif

// DrawRefCon constants
typedef short TListType;
const TListType 
	refNormal = -50,
	refLayers = -49,		// obsolete.
	refMoveSymbols = -48,
	refRecFld = -47,
	refTmp= -46,			// obsolete.
	refCheck = -45,		// obsolete.
	refSICN = -44,		// obsolete.
	refTiny = -43,
	
	refLarge = -42,
	refSmall = -41,
	refLargeCheck = -40,
	refSmallCheck = -39
;


typedef short ItemStatus;
const ItemStatus
	kActive = 2,
	kCheckOn = 4;

/* Same as a Str31 but doesn't depend <MacTypes.h>
 */
typedef unsigned char TListStr[32];

typedef struct TGeneralListEntry {
	long						data;
	TListStr					name;
	ItemStatus 					status;
} TGeneralListEntry;
typedef TGeneralListEntry 	   *TPGeneralListEntry;



#if _MINICAD_EXTERNAL_ && __cplusplus

////////////////////////////////
//
//	Standard entry point functions
//
////////////////////////////////
//
// These eliminate a lot of the low level entry and exit code associated with
// implementing a VectorWorks SDK Plug-In. To use these entry points, the file
// MenuMain.cpp, ToolMain.cpp, ParametricMain.cpp or LibraryMain.cpp should be added to your
// project (Mac or Windows) and the following function(s) should be implemented 
// in your code as the entry points for your Plug-In.
//
// NOTE - these standard entry point functions use C++ calling conventions.


////////////////////////////////
//
//	Menu entry points
//
////////////////////////////////

long MenuMain(long action, long message1, long &userData);
// MenuMain gets most mainstream menu messages

long DangerousMenuMain(long action, long message1, long &userData);
// DangerousMenuMain gets called with the kMenuCheckHilite & kMenuNotify messages
// the menu's resource file is NOT OPEN when these are called, so many
// normal operations, like calling across segments can cause a crash.
//
// DangerousMenuMain should typically just return 0. If you implement DangerousMenuMain
// be aware that your resource file is NOT OPEN when this function is called. If you make
// cross-segment calls on the Mac or read resources on Mac or Windows you will crash or fail.
//


////////////////////////////////
//
//	Tool entry points
//
////////////////////////////////
//
// Similar entry point for tools. Tools have no dangerous actions, so there is no need for
// a DangerousToolMain function.
//

long ToolMain(long action, long message1, long message2, long &userData);


////////////////////////////////
//
//	Plug-in Object entry points
//
////////////////////////////////
//
// Similar entry point for parametrics. Parametrics have no dangerous actions, so there is no need for
// a DangerousParametricMain function.
//

long ParametricMain(long action, MCObjectHandle parmHand, long message, long &userData);


////////////////////////////////
//
//	Plug-in Library entry points
//
////////////////////////////////
//
// JAK 5/31/00
//
long PluginLibraryMain(long routineSelector, void* argTable, long &userData);


#endif

#endif
