//
//	Copyright (c) 2006 Nemetschek North America 
//	All Rights Reserved.
//
// Created by Vladislav Stanev

#ifndef VECTORWORKS_PLUGIN_TOOL_APPLICATION__H
#define VECTORWORKS_PLUGIN_TOOL_APPLICATION__H

#pragma once

#define _VWFC_LEGACY
#include "VWFC/PluginSupport/VWExtensions.h"
#include "VWFC/PluginSupport/VWPluginToolCustomBar.h"

namespace VWFC
{
	namespace PluginSupport
	{
		// ---------------------------------------------------------------------------
		class VWPluginToolApp
		{
		public:
									VWPluginToolApp();
			virtual					~VWPluginToolApp();

			void					SetToolModeBarProvider(IToolModeBarProvider* pModeBarProvider);

			long					GetResult() const;
			void					SetResult(long result);

		public:
			virtual bool			InitializeApp();
			virtual void			DeinitializeApp();

			virtual long			GetUserData() const;
			virtual void			SetUserData(long userData);

			virtual TXString		GetContextualHelpID() const;
			virtual	void			OnInitXProperties(const IToolPropertiesProvider* pPropsProvider);

			virtual void			OnSelectionChange();
			virtual void			OnCustomBar_Setup();
			virtual void			OnCustomBar_Setdown();
			virtual void			OnCustomBar_UpdateValues();
			virtual void			OnCustomBar_LockValue(size_t fieldIndex, const TXString& userString);
			virtual void			OnCustomBar_UnlockValue(size_t fieldIndex);

			// When tool is selected or resumed
			virtual bool			DoSetUp(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider);
			// When tool is deselected	or paused
			virtual void			DoSetDown(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider);
			virtual void			DoDrawScrMod();
			virtual void			DoUndrawScrMod();

			// init the tool data for creation of the specified object
			virtual void			OnToolInitByObject(MCObjectHandle hObject, const IToolModeBarProvider* pModeBarInitProvider);

			// When tool selected and curs in doc
			virtual long			GetCursor();
			// When app needs tool's status
			virtual TToolStatus		GetStatus(const IToolStatusProvider* pStatusProvider);

			// When mouse is clicked in drawing
			virtual void			PointAdded();
			// When app removes a point
			virtual void			PointRemoved(bool bClearAllPts);
			// Whenever tool needs to draw
			virtual void			Draw(IToolDrawProvider* pDrawProvider)		= 0;
			// When mouse moves in drawing
			virtual void			MouseMove();
			// When tool is done getting points
			virtual void			HandleComplete() = 0;

			// When mouse is clicked in mode bar
			virtual void			DoModeEvent(size_t modeGroupID, size_t newButtonID, size_t oldButtonID);
			// When tool is double-clicked
			virtual void			DoDoubleClick();
			// When double-click in drawing
			virtual bool			DrawingDoubleClick();
			// After every key event.
			virtual void			GenericStateChange();

			// The default event notificaiotn
			// if no virutal is present for an event, this function is called to allow the user to catch everything
			virtual long			OnDefaultEvent(long action, long message1, long message2);

		// tool points
		public:
			size_t					GetToolPointsCount() const;
			void					ClearAllToolPoints();
			VWPoint2D				GetToolPt2D(size_t ptIndex) const;
			VWPoint3D				GetToolPt3D(size_t ptIndex) const;
			bool					GetToolPtCurren2D(VWPoint2D& outPt) const;
			bool					GetToolPtCurren3D(VWPoint3D& outPt) const;
			bool					PopLastToolPt();

		// hot spot points
		public:
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID);
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID, const TXString& strTip);
			long					HotSpotTrack(bool& bOutIs3D, VWPoint3D& outLocation);
			void					HotSpotRemove(long hotSpotID);

		// snap geometry
		public:
			long					SnapGeometryAdd(MCObjectHandle hSnapGeometry);
			void					SnapGeometryRemove(long snapGeometryID);

		protected:
			IToolModeBarProvider*	fpModeBarProvider;
			long					fResult;
		};
	}
}

// ------------------------------------------------------------------------------
#define	INSTANTIATE_TOOL_APP(T)	VWFC::PluginSupport::VWPluginToolApp* CreateToolInstance()  { void* pRawMem = malloc( sizeof(T) ); return (T*) new (pRawMem) T(); }

#endif // VECTORWORKS_PLUGIN_TOOL_APPLICATION__H