//
//	Copyright  2006 Nemetschek North America 
//	All Rights Reserved.
//
// Created by Vladislav Stanev


#ifndef VW_RECORD_FORMAT_OBJECT__H
#define VW_RECORD_FORMAT_OBJECT__H

#pragma once

namespace VWFC
{
	using namespace Tools;
	
	namespace VWObjects
	{
		class VWRecordFormatObj : public VWObject
		{
		public:
								VWRecordFormatObj(MCObjectHandle hFormatHandle);
								VWRecordFormatObj(const TXString& formatName);
								VWRecordFormatObj(const VWRecordFormatObj& src);
			virtual				~VWRecordFormatObj();

			VWRecordFormatObj&	operator=(const VWRecordFormatObj& src);
	
		public:
			TXString			GetFormatName() const;
			bool				IsParametric() const;
			void				SetParametric(bool bIsParametric);
			bool				GetFormatVisibility() const;
			void				SetFormatVisibility(bool bVisibility);

		// parameters
		public:
			size_t				AddParamLong(const TXString& name, long value, bool bUpdateExistingRecords = true);
			size_t				AddParamBool(const TXString& name, bool value, bool bUpdateExistingRecords = true);
			size_t				AddParamReal(const TXString& name, double value, bool bUpdateExistingRecords = true);
			size_t				AddParamString(const TXString& name, const TXString& value, bool bUpdateExistingRecords = true);
			size_t				AddParamClassesPopUp(const TXString& name, const TXString& value, bool bUpdateExistingRecords = true);

			size_t				GetParamsCount() const;
			TXString			GetParamName(size_t paramIndex) const;
			TXString			GetParamLocalizedName(size_t paramIndex) const;
			// returns ((size_t) -1) if not found
			size_t				GetParamIndex(const TXString& paramName) const;

			// universal param name based
			EFieldStyle			GetParamStyle(const TXString& univParamName) const;
			TXString			GetParamValue(const TXString& univParamName) const;
			void				SetParamValue(const TXString& univParamName, const TXString& value);
			long				GetParamLong(const TXString& univParamName) const;
			void				SetParamLong(const TXString& univParamName, long value);
			bool				GetParamBool(const TXString& univParamName) const;
			void				SetParamBool(const TXString& univParamName, bool value);
			double				GetParamReal(const TXString& univParamName) const;
			void				SetParamReal(const TXString& univParamName, double value);
			TXString			GetParamString(const TXString& univParamName) const;
			void				SetParamString(const TXString& univParamName, const TXString& value);

			// index based
			// NOTE: Recomended use of universal parameter name based (above) functions because they are safer
			// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			EFieldStyle			/*Use univParamName funcsion instead*/	GetParamStyle(size_t paramIndex) const;
			TXString			/*Use univParamName funcsion instead*/	GetParamValue(size_t paramIndex) const;
			void				/*Use univParamName funcsion instead*/	SetParamValue(size_t paramIndex, const TXString& value);
			long				/*Use univParamName funcsion instead*/	GetParamLong(size_t paramIndex) const;
			void				/*Use univParamName funcsion instead*/	SetParamLong(size_t paramIndex, long value);
			bool				/*Use univParamName funcsion instead*/	GetParamBool(size_t paramIndex) const;
			void				/*Use univParamName funcsion instead*/	SetParamBool(size_t paramIndex, bool value);
			double				/*Use univParamName funcsion instead*/	GetParamReal(size_t paramIndex) const;
			void				/*Use univParamName funcsion instead*/	SetParamReal(size_t paramIndex, double value);
			TXString			/*Use univParamName funcsion instead*/	GetParamString(size_t paramIndex) const;
			void				/*Use univParamName funcsion instead*/	SetParamString(size_t paramIndex, const TXString& value);

		// Tools
		public:
			// Return true if the specified object handle is record format
			static bool				IsRecordFormatObject(MCObjectHandle hObj);
			static bool				IsRecordFormatObject(MCObjectHandle hObj, const char* szName);
			static MCObjectHandle	GetRecordFormat(const char* szName);

			static void				DeleteRecordFormat(MCObjectHandle hRecFormat);

		protected:
			size_t				GetParamIndexByUnivName(const TXString& univParamName) const;

		protected:
			typedef std::map<TXString, size_t>		TUnviNameKeyMap;
			TUnviNameKeyMap		fmapUnivNameKey;
		};
	}
}

#endif // VW_RECORD_FORMAT_OBJECT__H