//
//	Copyright (c) 2005 Nemetschek North America 
//	All Rights Reserved.
//
// Created by Vladislav Stanev

#include "StdHeaders.h"

#include "VWFC/PluginSupport/VWExtensions.h"
#include "VWFC/PluginSupport/VWExtensionTool.h"
#include "VWFC/PluginSupport/VWPluginToolProviders.h"

using namespace VWFC::PluginSupport;
using namespace VWFC::Math;

// --------------------------------------------------------------------------------------------------------
VWExtensionTool::VWExtensionTool(	CallBackPtr cbp,
									const SToolDef& toolDef)
	: fCBP( cbp )
	, fToolDef( toolDef )
	, fLegacyEventSink( NULL )
	, fProtectionObtained( false )
	, fHasProtection( false )
	, fProtectionSink( NULL )
{
	gCBP			= fCBP;
	fstrTitle.Load( fToolDef.fTitle.fListID, fToolDef.fTitle.fIndex );
	fstrCategory.Load( fToolDef.fCategory.fListID, fToolDef.fCategory.fIndex );
	fstrMessage.Load( fToolDef.fMessage.fListID, fToolDef.fMessage.fIndex );
	if ( fToolDef.fHelpText.fID > 0 ) {
		VERIFYN( kVStanev, VWFC::Tools::CSTRResLoader::Instance().LoadText( fToolDef.fHelpText.fID, fstrHelp ) );
	}

	if ( toolDef.fParametricName ) {
		fstrParametricName	= toolDef.fParametricName;
	}

	DSTOPIF(	(fstrParametricName.IsEmpty() && fToolDef.fToolType != eExtensionToolType_Normal)
			||	(!fstrParametricName.IsEmpty() && fToolDef.fToolType == eExtensionToolType_Normal)
			,
			(kVStanev, "No 'parametric name' means toolType_Normal. Otherwise tool type should reflect the type of the specified parametric object."));

	if ( toolDef.fContextualHelpID ) {
		fstrContextualHelpID	= toolDef.fContextualHelpID;
	}
}

VWExtensionTool::~VWExtensionTool()
{
}

void VWExtensionTool::OnRefCountZero()
{
	if ( fLegacyEventSink ) {
		delete fLegacyEventSink;
		fLegacyEventSink	= NULL;
	}

	if ( fProtectionSink ) {
		delete fProtectionSink;
		fProtectionSink	= NULL;
	}

	// unload resources
	::GS_VWUnloadInterfaceResource( this );
}

IEventSink* VWExtensionTool::QueryEventSink(const TSinkIID& iid)
{
	if ( fProtectionObtained == false ) {
		fHasProtection			= this->GetProtectionSinkIID( fProtectionSinkIID );
		fProtectionObtained		= true;
	}

	IEventSink*	pResult	= NULL;
	if ( IID_LegacyEventSink == iid ) {
		if ( fLegacyEventSink == NULL ) {
			fLegacyEventSink	= this->CreateLegacyEventSink( this );
			fLegacyEventSink->AddRef();
			VWTool_EventSink*	vwfcSink	= dynamic_cast<VWTool_EventSink*>( fLegacyEventSink );
			if ( vwfcSink ) {
				vwfcSink->SetUniversalName( this->GetUniversalName() );
			}
			else {
				VWToolDefault_EventSink*	defaultSink	= dynamic_cast<VWToolDefault_EventSink*>( fLegacyEventSink );
				if ( defaultSink ) {
					defaultSink->Init( this->GetUniversalName() );
				}
			}
		}
		pResult	= fLegacyEventSink;
	}
	else if ( fHasProtection && fProtectionSinkIID == iid ) {
		if ( fProtectionSink == NULL ) {
			fProtectionSink	= this->CreateProtectionSink( this );
			if ( fProtectionSink == NULL ) {
				fHasProtection	= false;
			}
		}

		if ( fProtectionSink ) {
			fProtectionSink->AddRef();
			pResult	= fProtectionSink;
		}
	}

	return pResult;
}

bool VWExtensionTool::GetProtectionSinkIID(TSinkIID& outIID)
{
	return false;
}

IProviderProtection* VWExtensionTool::CreateProtectionSink(IVWUnknown* parent)
{
	return NULL;
}

EExtensionToolType VWExtensionTool::GetType()
{
	return fToolDef.fToolType;
}

const TXString& VWExtensionTool::GetTitle()
{
	return fstrTitle;
}

const TXString& VWExtensionTool::GetCategory()
{
	return fstrCategory;
}

const TXString& VWExtensionTool::GetHelpText()
{
	return fstrHelp;
}

const TXString& VWExtensionTool::GetMessage()
{
	return fstrMessage;
}

const TXString& VWExtensionTool::GetContextualHelpID()
{
	return fstrContextualHelpID;
}

const TXString&	VWExtensionTool::GetParametricName()
{
	return fstrParametricName;
}

bool VWExtensionTool::GePickAndUpdate()
{
	return fToolDef.fPickAndUpdate == ToolDef::pickAndUpdate;
}

bool VWExtensionTool::GetNeedScreenPlane()
{
	return fToolDef.fNeedScreenPlane == ToolDef::needsScreenPlane;
}

bool VWExtensionTool::GetNeed3DProjection()
{
	return fToolDef.fNeed3DView == ToolDef::needs3DView;
}

bool VWExtensionTool::GetUse2DCursor()
{
	return fToolDef.fUseCursor == ToolDef::use2DCursor;
}

bool VWExtensionTool::GetConstrainCursor()
{
	return fToolDef.fConstrainCursor == ToolDef::constrainCursor;
}

bool VWExtensionTool::GetNeedPerspective()
{
	return fToolDef.fNeedPerspective == ToolDef::needsPerspective;
}

bool VWExtensionTool::GetShowScreenHints()
{
	return fToolDef.fShowScreenHints == ToolDef::showScreenHints;
}

bool VWExtensionTool::GetNeedsPlanarContext()
{
	return fToolDef.fNeedsPlanarContext == ToolDef::needsPlanarContext;
} 

Sint16 VWExtensionTool::GetIconID()
{
	return fToolDef.fIconID;
}

Sint16 VWExtensionTool::GetDefaultCursorID()
{
	return fToolDef.fDefaultCursorID;
}

Sint16 VWExtensionTool::GetWaitMoveDistance()
{
	return fToolDef.fWaitMoveDistance;
}

Sint16 VWExtensionTool::GetConstraintFlags()
{
	return fToolDef.fConstraintFlags;
}

EToolBarDisplay VWExtensionTool::GetBarDisplay()
{
	return fToolDef.fBarDisplay;
}

Sint16 VWExtensionTool::GetMinimumCompatibleVersion()
{
	return fToolDef.fMinimumCompatibleVersion;
}

void  VWExtensionTool::GetVersionInfo(Uint16& outCreated, Uint16& outModified, Uint16& outRetired)
{
	outCreated		= fToolDef.fVersionCreated;
	outModified		= fToolDef.fVersoinModified;
	outRetired		= fToolDef.fVersoinRetired;
}


// --------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------
// predefine these hidden memory managment functions
namespace VWFC
{
	void	AllocateMemoryManagers(VWFC::MemoryManager::MManager** ppOutOldManager, VWFC::MemoryManager::MManager** ppOutOldVWManager);
	void	FreeMemoryManagers(VWFC::MemoryManager::MManager* pOldManager, VWFC::MemoryManager::MManager* pOldVWManager);
}

// --------------------------------------------------------------------------------------------------------
VWTool_EventSink::VWTool_EventSink(IVWUnknown* parent)
	: VCOMImpl<ILegacyEventSink>( parent )
{
	fpModeBarProvider		= NULL;
	fResult					= 0;
}

VWTool_EventSink::~VWTool_EventSink()
{
}

void VWTool_EventSink::SetUniversalName(const TXString& name)
{
	fUnivName	= name;
}

long VWTool_EventSink::LegacyMain(long action, long message1, long message2)
{
	long	reply	= kToolEventHandled;

	// create the memory namagers
	// and verify their presents
	VWFC::MemoryManager::MManager* pOldManager		= NULL;
	VWFC::MemoryManager::MManager* pOldVWManager	= NULL;
	VWFC::AllocateMemoryManagers( & pOldManager, & pOldVWManager );

	VWFC_VERIFY( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL );
	if ( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL ) {

		this->SetResult( kToolEventHandled );

		//DMSG(( kVStanev, "VWFC::ToolMain: action=%d\n", action ));
		IToolModeBarProvider*	savedModeBarProvider	= fpModeBarProvider;

		CModeBarProvider	modeBarProvider;
		fpModeBarProvider		= & modeBarProvider;

		switch (action) {
			case kToolDoSetup: {
					CModeBarInitProvider	modeBarInitProvider;

					reply	= this->DoSetUp( message1 == kSetupSetdownPauseRestoreFlag, & modeBarInitProvider ) ? this->GetResult() : kToolCouldNotSetUp;
				} break;

			case kToolDoSetDown: {
					CModeBarInitProvider	modeBarInitProvider;

					this->DoSetDown( message1 == kSetupSetdownPauseRestoreFlag, & modeBarInitProvider );
					reply	= this->GetResult();
				} break;

			case kToolDoDrawScrMod:
				this->DoDrawScrMod();
				reply	= this->GetResult();
				break;

			case kToolDoUndrawScrMod:
				this->DoUndrawScrMod();
				reply	= this->GetResult();
				break;

			case kOnToolInitXProperties: {
					CToolPropertiesProvider		propsProvider( message1 );

					this->OnInitXProperties( & propsProvider );
					reply	= this->GetResult();
				} break;

			case kToolSelectionChange:
				this->OnSelectionChange();
				reply	= this->GetResult();
				break;

			case kToolCustomBarEvent: {
					CustomBarEventData* pEventData = (CustomBarEventData*) message1;
					if ( pEventData ) {
						switch( pEventData->fCustomBarEventSelector ) {
							case kCustomBarSetup:
								this->OnCustomBar_Setup();
								break;
							case kCustomBarSetdown:
								this->OnCustomBar_Setdown();
								break;
							case kCustomBarUpdateValues:
								this->OnCustomBar_UpdateValues();
								break;
							case kCustomBarLockValue: {
								CustomBarLockValueData* pLockData = (CustomBarLockValueData*) pEventData->fData;
								if ( pLockData ) {
									this->OnCustomBar_LockValue( pLockData->fFieldIndex, pLockData->fUserLockString );
								}
							}break;
							case kCustomBarUnlockValue:
								this->OnCustomBar_UnlockValue( pEventData->fData );
								break;
						}
					}
					reply	= this->GetResult();
				} break;

			case kToolGetCursor:
				reply	= this->GetCursor();
				break;

			case kToolGetStatus: {
					CStatusProvider	statusProvider;

					reply	= this->GetStatus( & statusProvider );
				} break;

			case kToolPointAdded:
				this->PointAdded();
				reply	= this->GetResult();
				break;

			case kToolPointRemoved:
				this->PointRemoved( message1 == kClearingAllToolPoints );
				reply	= this->GetResult();
				break;

			case kToolDraw: {
					CDrawProvider	drawProvider;
					this->Draw( & drawProvider);
					reply	= this->GetResult();
				} break;

			case kToolMouseMove:
				this->MouseMove();
				reply	= this->GetResult();
				break;

			case kToolHandleComplete:
				this->HandleComplete();
				reply	= this->GetResult();
				break;

	//			case kToolDoInterface:		// DEPRECATED
	//				this->DoInterface( VWPoint2D( * ((WorldPt*)message1) ) );
	//				break;

			case kToolDoModeEvent: {
					size_t		groupID		= message1 > 0 ? (size_t)(message1-1) : 0;
					size_t		buttonID	= message2 > 0 ? (size_t)(message2-1) : 0;
					size_t		newButtonID	= buttonID & 0x0000FFFF;
					size_t		oldButtonID	= (buttonID & 0xFFFF0000) >> 24;

					this->DoModeEvent( groupID, newButtonID, oldButtonID );
					reply	= this->GetResult();
				} break;

			case kToolDoDoubleClick:
				this->DoDoubleClick();
				reply	= this->GetResult();
				break;

			case kToolDrawingDoubleClick:
				this->SetResult( kToolSwitchToCursor );
				reply	= this->DrawingDoubleClick() ? kToolCompleted : this->GetResult();
				break;

			case kToolGenericStateChange:
				this->GenericStateChange();
				break;

			case kToolDisplayContextualHelp:
				DSTOP((kVStanev, "we should not be here for module plug-ins"));
				break;

			case kToolInitByObject: {
					MCObjectHandle			hTheObject	= (MCObjectHandle) message1;
					CModeBarInitProvider	modeBarInitProvider;

					this->OnToolInitByObject( hTheObject, & modeBarInitProvider );
				} break;
			default:
				reply = this->OnDefaultEvent( action, message1, message2 );
				break;
		}

		// restore the mode provider
		fpModeBarProvider		= savedModeBarProvider;

		// dump local memory leaks
		if ( VWFC::GetMManager() ) {
			VWFC::GetMManager()->DumpMemory( fUnivName );
		}
	}

	// release the memory managers
	VWFC::FreeMemoryManagers( pOldManager, pOldVWManager );

	return reply;
}

long VWTool_EventSink::GetResult() const
{
	return fResult;
}

void VWTool_EventSink::SetResult(long result)
{
	fResult	= result;
}

void VWTool_EventSink::OnInitXProperties(const IToolPropertiesProvider* pPropsProvider)
{
}

void VWTool_EventSink::OnSelectionChange()
{
}

void VWTool_EventSink::OnCustomBar_Setup()
{
}

void VWTool_EventSink::OnCustomBar_Setdown()
{
}

void VWTool_EventSink::OnCustomBar_UpdateValues()
{
}

void VWTool_EventSink::OnCustomBar_LockValue(size_t fieldIndex, const TXString& userString)
{
}

void VWTool_EventSink::OnCustomBar_UnlockValue(size_t fieldIndex)
{
}

bool VWTool_EventSink::DoSetUp(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider)
{
	return true;
}

void VWTool_EventSink::DoSetDown(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider)
{
}

void VWTool_EventSink::DoDrawScrMod()
{
}

void VWTool_EventSink::DoUndrawScrMod()
{
}

void VWTool_EventSink::OnToolInitByObject(MCObjectHandle hObject, const IToolModeBarProvider* pModeBarInitProvider)
{
}

long VWTool_EventSink::GetCursor()
{
	return 0;
}

TToolStatus VWTool_EventSink::GetStatus(const IToolStatusProvider* pStatusProvider)
{
	return pStatusProvider->GetOnePointToolStatus();
}

void VWTool_EventSink::PointAdded()
{
}

void VWTool_EventSink::PointRemoved(bool bClearAllPts)
{
}

void VWTool_EventSink::MouseMove()
{
}

void VWTool_EventSink::DoModeEvent(size_t modeGroupID, size_t newButtonID, size_t oldButtonID)
{
}

void VWTool_EventSink::DoDoubleClick()
{
}

bool VWTool_EventSink::DrawingDoubleClick()
{
	return true;
}

void VWTool_EventSink::GenericStateChange()
{
}

size_t VWTool_EventSink::GetToolPointsCount() const
{
	return ::GS_GetNumToolPts( gCBP );
}

void VWTool_EventSink::ClearAllToolPoints()
{
	VWFC_VERIFY( ::GS_ClearAllToolPts( gCBP ) );
}

VWPoint2D VWTool_EventSink::GetToolPt2D(size_t ptIndex) const
{
	VWFC_ASSERT( ptIndex < this->GetToolPointsCount() );

	WorldPt		pt;
	VWFC_VERIFY( ::GS_GetToolPt2D( gCBP, DemoteTo<long>( kVStanev, ptIndex ), pt ) );
	return pt;
}

VWPoint3D VWTool_EventSink::GetToolPt3D(size_t ptIndex) const
{
	VWFC_ASSERT( ptIndex < this->GetToolPointsCount() );

	WorldPt3	pt;
	VWFC_VERIFY( ::GS_GetToolPt3D( gCBP, DemoteTo<long>( kVStanev, ptIndex ), pt ) );
	return pt;
}

bool VWTool_EventSink::GetToolPtCurren2D(VWPoint2D& outPt) const
{
	WorldPt		pt;
	bool		ok	= ::GS_GetToolPtCurrent2D( gCBP, pt );
	outPt			= pt;
	return ok;
}

bool VWTool_EventSink::GetToolPtCurren3D(VWPoint3D& outPt) const
{
	WorldPt3	pt;
	bool		ok	= ::GS_GetToolPtCurrent3D( gCBP, pt ) ? true : false;
	outPt			= pt;
	return ok;
}

bool VWTool_EventSink::PopLastToolPt()
{
	return GS_PopLastToolPt( gCBP ) ? true : false;
}


/*
	extern "C" Boolean GS_CALLBACK GS_GetToolPtCurrentViewPt(CallBackPtr, ViewPt& outToolPt);
	//	Returns in outToolPt the current view space point. 
	extern "C" Boolean GS_CALLBACK GS_GetToolPtCurrentWorkingPlane(CallBackPtr, WorldPt3& outToolPt);
	//	Returns in outToolPt the current 3D working plane point. 
	extern "C" Boolean GS_CALLBACK GS_GetToolPtViewPoint(CallBackPtr, long inIndex, ViewPt& outToolPoint);
	//		Gets  the specified view space ToolPoint .  The index  is a zero based index .  False is returned if
	//		the index is out of range 0 <= index< GS_GetNumToolPoints.
	extern "C" Boolean GS_CALLBACK GS_GetToolPtWorkingPlane(CallBackPtr, long inIndex, WorldPt3& outToolPoint);
	//		Gets  the specified 3D working plane ToolPoint .  The index  is a zero based index .  False is
	//		returned if the index is out of range 0 <= index< GS_GetNumToolPoints.
*/

long VWTool_EventSink::HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID)
{
	return gSDK->AddHotSpot( type, pt, cursorID, "" );
}

long VWTool_EventSink::HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID, const TXString& strTip)
{
	return gSDK->AddHotSpot( type, pt, cursorID, strTip );
}

long VWTool_EventSink::HotSpotTrack(bool& bOutIs3D, VWPoint3D& outLocation)
{
	Boolean		is3d		= false;
	WorldPt3	location;
	long		hotSpotID	= gSDK->TrackHotSpot( is3d, location );

	bOutIs3D				= is3d ? true : false;
	outLocation				= location;
	return hotSpotID;
}

void VWTool_EventSink::HotSpotRemove(long hotSpotID)
{
	::GS_RemoveHotSpot( gCBP, hotSpotID );
}

long VWTool_EventSink::SnapGeometryAdd(MCObjectHandle hSnapGeometry)
{
	return ::GS_AddToolSnapGeometry( gCBP, hSnapGeometry );
}

void VWTool_EventSink::SnapGeometryRemove(long snapGeometryID)
{
	::GS_RemoveToolSnapGeometry( gCBP, snapGeometryID );
}


/*
	extern "C" void GS_CALLBACK GS_TrackTool(CallBackPtr, MCObjectHandle& overObject, short& overPart, long& code);
			Used during the kToolDoPick case of a tool definition.  Returns in overObject the object the cursor
			is currently over (or nil if the cursor is not over any object) and in overPart one of cursOnNothing
			(object is not over any snappable object), cursOnObject (cursor is over an object), or cursOnHandle
			(object is over a selected objects handle).  TrackTool only responds to snappable objects, i.e.
			objects that are on a currently editable or snappable layer. code returns a long that is specific to
			each overPart message.

	extern "C" Boolean GS_CALLBACK GS_RunTemporary2DTool(CallBackPtr cbp, long& ioUserData, ToolDefProcPtr inToolProc);
			RunTemporary2DTool runs a temporary tool as defined by the action handleing of inToolProc. 

	extern "C" Boolean GS_CALLBACK GS_RunTemporary3DTool(CallBackPtr cbp, long& ioUserData, ToolDefProcPtr inToolProc);
			RunTemporary3DTool runs a temporary tool as defined by the action handleing of inToolProc. 
*/
//////////////////////////////////////////////
/*
void VWTool_EventSink::SetToolPersistentDraw()
{
	// we here allow only one persistent draw callback
	VWFC_ASSERT( fPersistentDrawID == 0 );
	if ( fPersistentDrawID == 0 ) {
		fPersistentDrawID	= ::GS_AddToolPersistentDraw( gCBP, _PersistentDrawCallback, this );
	}
}

void VWTool_EventSink::RemoveToolPersistentDraw()
{
	::GS_RemoveToolPersistentDraw( gCBP, fPersistentDrawID );
}

void VWTool_EventSink::_PersistentDrawCallback(CallBackPtr cbp, void* pEnv)
{
	VWTool_EventSink*	pThis	= (VWTool_EventSink*) pEnv;
	VWFC_ASSERT( pThis );
	if ( pThis ) {
		CDrawProvider	 drawProvider;

		pThis->OnToolPersistentDraw( & drawProvider );
	}
}

void VWTool_EventSink::OnToolPersistentDraw(IToolDrawProvider* pDrawProvider)
{
}*/

long VWTool_EventSink::OnDefaultEvent(long action, long message1, long message2)
{
	return 0;
}

//////////////////////////////////////////////
/*
	extern "C" void GS_CALLBACK GS_Interactive2DDraw(CallBackPtr cbp, short options, short shiftConsMode, GS_I2DDrawProcPtr drawProc, void *env);
//			Interactive2DDraw is called when drawing interactively.  To use it, you must define all drawing to
//			be done as a function of the worldspace start and endpoints of a drag operation.
//			the prototype for Interactive2DDraw is :
//					
//			Interactive2DDraw(CallBackPtr cbp, short options, short shiftConsMode, I2DDrawProcPtr drawProc, void
//			*env);
//					
//			where drawProc is the function that does the drawing.  Interactive2DDraw takes care of 
//			the draw and undraw for you as well as respond to any constraints set with shiftConsMode 
//			or the constaint set in the TDef.


	extern "C" void GS_CALLBACK GS_GetCoords(CallBackPtr, WorldPt& p, Point& m);
//			Returns the final mouse position after an Interactive2DDraw call, both in world and view coordinates.
		*/


//////////////////////////////////////////////


// --------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------------------------------
VWToolEmpty_EventSink::VWToolEmpty_EventSink(IVWUnknown* parent)
	: VCOMImpl<ILegacyEventSink>( parent )
{
}

VWToolEmpty_EventSink::~VWToolEmpty_EventSink()
{
}

long VWToolEmpty_EventSink::LegacyMain(long action, long message1, long message2)
{
	action;
	message1;
	message2;
	return 0;
}


// --------------------------------------------------------------------------------------------------------
VWToolDefault_EventSink::VWToolDefault_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VCOMImpl<ILegacyEventSink>( parent )
{
	fUserData		= 0;
	fObjectName		= objectName;
	fToolType		= eDefaultToolType_Point;
	fIsStandardTool	= true;
}

VWToolDefault_EventSink::~VWToolDefault_EventSink()
{
	gSDK->CallDefaultToolN( fToolType, fIsStandardTool, kToolDisposeGlobals, NULL, NULL, fUserData );
}

void VWToolDefault_EventSink::Init(const TXString& universalName)
{
	fUniversalName	= universalName;
}

// --------------------------------------------------------------------------------------------------------
VWToolDefaultPoint_EventSink::VWToolDefaultPoint_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VWToolDefault_EventSink( parent, objectName )
{
	fToolType		= eDefaultToolType_Point;
}

VWToolDefaultPoint_EventSink::~VWToolDefaultPoint_EventSink()
{
}

void VWToolDefaultPoint_EventSink::Init(const TXString& universalName)
{
	VWToolDefault_EventSink::Init( universalName );

	InternalName	theObjectName;
	fObjectName.CopyInto( theObjectName );

	if ( ! fIsStandardTool ) {
		gSDK->CallDefaultToolN( fToolType, false, kToolInitGlobals, (long) & theObjectName, (long) & fToolInfo, fUserData );
	}
	else {
		gSDK->CallDefaultToolN( fToolType, true, kToolInitGlobals, (long) & theObjectName, NULL, fUserData );
	}
}

const SDefaultPointToolInfo& VWToolDefaultPoint_EventSink::GetToolInfo() const
{
	return fToolInfo;
}

void VWToolDefaultPoint_EventSink::SetToolInfo(const SDefaultPointToolInfo& info)
{
	fToolInfo		= info;
	fIsStandardTool	= false;
}

long VWToolDefaultPoint_EventSink::LegacyMain(long action, long message1, long message2)
{
	if ( ! fIsStandardTool ) {
		if ( action == kToolDoSetup ) {
			message2	= (long) & fToolInfo;
		}
	}

	return gSDK->CallDefaultToolN( fToolType, fIsStandardTool, action, message1, message2, fUserData );
}


// --------------------------------------------------------------------------------------------------------
VWToolDefaultLine_EventSink::VWToolDefaultLine_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VWToolDefault_EventSink( parent, objectName )
{
	fToolType		= eDefaultToolType_Line;
}

VWToolDefaultLine_EventSink::~VWToolDefaultLine_EventSink()
{
}

void VWToolDefaultLine_EventSink::Init(const TXString& universalName)
{
	VWToolDefault_EventSink::Init( universalName );

	InternalName	theObjectName;
	fObjectName.CopyInto( theObjectName );

	if ( ! fIsStandardTool ) {
		gSDK->CallDefaultToolN( fToolType, false, kToolInitGlobals, (long) & theObjectName, (long) & fToolInfo, fUserData );
	}
	else {
		gSDK->CallDefaultToolN( fToolType, true, kToolInitGlobals, (long) & theObjectName, NULL, fUserData );
	}
}

const SDefaultLineToolInfo& VWToolDefaultLine_EventSink::GetToolInfo() const
{
	return fToolInfo;
}

void VWToolDefaultLine_EventSink::SetToolInfo(const SDefaultLineToolInfo& info)
{
	fToolInfo		= info;
	fIsStandardTool	= false;
}

long VWToolDefaultLine_EventSink::LegacyMain(long action, long message1, long message2)
{
	if ( ! fIsStandardTool ) {
		if ( action == kToolDoSetup ) {
			message2	= (long) & fToolInfo;
		}
	}

	return gSDK->CallDefaultToolN( fToolType, fIsStandardTool, action, message1, message2, fUserData );
}


// --------------------------------------------------------------------------------------------------------
VWToolDefaultRect_EventSink::VWToolDefaultRect_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VWToolDefault_EventSink( parent, objectName )
{
	fToolType		= eDefaultToolType_Rect;
}

VWToolDefaultRect_EventSink::~VWToolDefaultRect_EventSink()
{
}

void VWToolDefaultRect_EventSink::Init(const TXString& universalName)
{
	VWToolDefault_EventSink::Init( universalName );

	InternalName	theObjectName;
	fObjectName.CopyInto( theObjectName );

	if ( ! fIsStandardTool ) {
		gSDK->CallDefaultToolN( fToolType, false, kToolInitGlobals, (long) & theObjectName, (long) & fToolInfo, fUserData );
	}
	else {
		gSDK->CallDefaultToolN( fToolType, true, kToolInitGlobals, (long) & theObjectName, NULL, fUserData );
	}
}

const SDefaultRectToolInfo& VWToolDefaultRect_EventSink::GetToolInfo() const
{
	return fToolInfo;
}

void VWToolDefaultRect_EventSink::SetToolInfo(const SDefaultRectToolInfo& info)
{
	fToolInfo		= info;
	fIsStandardTool	= false;
}

long VWToolDefaultRect_EventSink::LegacyMain(long action, long message1, long message2)
{
	if ( ! fIsStandardTool ) {
		if ( action == kToolDoSetup ) {
			message2	= (long) & fToolInfo;
		}
	}

	return gSDK->CallDefaultToolN( fToolType, fIsStandardTool, action, message1, message2, fUserData );
}


// --------------------------------------------------------------------------------------------------------
VWToolDefault2DPoly_EventSink::VWToolDefault2DPoly_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VWToolDefault_EventSink( parent, objectName )
{
	fToolType		= eDefaultToolType_Poly2D;
}

VWToolDefault2DPoly_EventSink::~VWToolDefault2DPoly_EventSink()
{
}

void VWToolDefault2DPoly_EventSink::Init(const TXString& universalName)
{
	VWToolDefault_EventSink::Init( universalName );

	InternalName	theObjectName;
	fObjectName.CopyInto( theObjectName );

	if ( ! fIsStandardTool ) {
		gSDK->CallDefaultToolN( fToolType, false, kToolInitGlobals, (long) & theObjectName, (long) & fToolInfo, fUserData );
	}
	else {
		gSDK->CallDefaultToolN( fToolType, true, kToolInitGlobals, (long) & theObjectName, NULL, fUserData );
	}
}

const SDefault2DPolyToolInfo& VWToolDefault2DPoly_EventSink::GetToolInfo() const
{
	return fToolInfo;
}

void VWToolDefault2DPoly_EventSink::SetToolInfo(const SDefault2DPolyToolInfo& info)
{
	fToolInfo		= info;
	fIsStandardTool	= false;
}

long VWToolDefault2DPoly_EventSink::LegacyMain(long action, long message1, long message2)
{
	if ( ! fIsStandardTool ) {
		if ( action == kToolDoSetup ) {
			message2	= (long) & fToolInfo;
		}
	}

	return gSDK->CallDefaultToolN( fToolType, fIsStandardTool, action, message1, message2, fUserData );
}


// --------------------------------------------------------------------------------------------------------
VWToolDefault3DPoly_EventSink::VWToolDefault3DPoly_EventSink(IVWUnknown* parent, const TXString& objectName)
	: VWToolDefault_EventSink( parent, objectName )
{
	fToolType		= eDefaultToolType_Poly3D;
}

VWToolDefault3DPoly_EventSink::~VWToolDefault3DPoly_EventSink()
{
}

void VWToolDefault3DPoly_EventSink::Init(const TXString& universalName)
{
	VWToolDefault_EventSink::Init( universalName );

	InternalName	theObjectName;
	fObjectName.CopyInto( theObjectName );

	if ( ! fIsStandardTool ) {
		gSDK->CallDefaultToolN( fToolType, false, kToolInitGlobals, (long) & theObjectName, (long) & fToolInfo, fUserData );
	}
	else {
		gSDK->CallDefaultToolN( fToolType, true, kToolInitGlobals, (long) & theObjectName, NULL, fUserData );
	}
}

const SDefault3DPolyToolInfo& VWToolDefault3DPoly_EventSink::GetToolInfo() const
{
	return fToolInfo;
}

void VWToolDefault3DPoly_EventSink::SetToolInfo(const SDefault3DPolyToolInfo& info)
{
	fToolInfo		= info;
	fIsStandardTool	= false;
}

long VWToolDefault3DPoly_EventSink::LegacyMain(long action, long message1, long message2)
{
	if ( ! fIsStandardTool ) {
		if ( action == kToolDoSetup ) {
			message2	= (long) & fToolInfo;
		}
	}

	return gSDK->CallDefaultToolN( fToolType, fIsStandardTool, action, message1, message2, fUserData );
}


